"""Sparse polynomial rings."""

import functools
import math
import operator

from ..core import Expr, Symbol, oo
from ..core import symbols as _symbols
from ..core import sympify
from ..core.compatibility import is_sequence
from ..core.sympify import CantSympify
from ..domains.compositedomain import CompositeDomain
from ..domains.domainelement import DomainElement
from ..domains.ring import Ring
from ..ntheory import multinomial_coefficients
from ..ntheory.modular import symmetric_residue
from ..utilities.magic import pollute
from .compatibility import IPolys
from .constructor import construct_domain
from .densebasic import dmp_from_dict, dmp_to_dict
from .euclidtools import _GCD
from .heuristicgcd import heugcd
from .modulargcd import func_field_modgcd, modgcd
from .monomials import Monomial
from .orderings import lex
from .polyconfig import query
from .polyerrors import (CoercionFailed, DomainError, ExactQuotientFailed,
                         GeneratorsError, GeneratorsNeeded, HeuristicGCDFailed,
                         MultivariatePolynomialError, PolynomialDivisionFailed,
                         PolynomialError)
from .polyoptions import Domain as DomainOpt
from .polyoptions import Order as OrderOpt
from .polyoptions import build_options
from .polyutils import _dict_reorder, _parallel_dict_from_expr, expr_from_dict


__all__ = 'PolynomialRing', 'ring', 'sring', 'vring'


def ring(symbols, domain, order=lex):
    """Construct a polynomial ring returning ``(ring, x_1, ..., x_n)``.

    Parameters
    ==========

    symbols : str, Symbol/Expr or sequence of str, Symbol/Expr (non-empty)
    domain : :class:`~diofant.domains.domain.Domain` or coercible
    order : :class:`~diofant.polys.polyoptions.Order` or coercible, optional, defaults to ``lex``

    Examples
    ========

    >>> R, x, y, z = ring('x y z', ZZ)
    >>> R
    ZZ[x,y,z]
    >>> x + y + z
    x + y + z

    """
    _ring = PolynomialRing(domain, symbols, order)
    return (_ring,) + _ring.gens


def vring(symbols, domain, order=lex):
    """Construct a polynomial ring and inject ``x_1, ..., x_n`` into the global namespace.

    Parameters
    ==========

    symbols : str, Symbol/Expr or sequence of str, Symbol/Expr (non-empty)
    domain : :class:`~diofant.domains.domain.Domain` or coercible
    order : :class:`~diofant.polys.polyoptions.Order` or coercible, optional, defaults to ``lex``

    Examples
    ========

    >>> vring('x y z', ZZ)
    ZZ[x,y,z]
    >>> x + y + z
    x + y + z

    """
    _ring = PolynomialRing(domain, symbols, order)
    pollute([sym.name for sym in _ring.symbols], _ring.gens)
    return _ring


def sring(exprs, *symbols, **options):
    """Construct a ring deriving generators and domain from options and input expressions.

    Parameters
    ==========

    exprs : :class:`~diofant.core.expr.Expr` or sequence of :class:`~diofant.core.expr.Expr` (sympifiable)
    symbols : sequence of :class:`~diofant.core.symbol.Symbol`/:class:`~diofant.core.expr.Expr`
    options : keyword arguments understood by :class:`~diofant.polys.polyoptions.Options`

    Examples
    ========

    >>> R, f = sring(x + 2*y + 3*z)
    >>> R
    ZZ[x,y,z]
    >>> f
    x + 2*y + 3*z

    """
    single = False

    if not is_sequence(exprs):
        exprs, single = [exprs], True

    exprs = list(map(sympify, exprs))
    opt = build_options(symbols, options)

    reps, opt = _parallel_dict_from_expr(exprs, opt)

    if opt.domain is None:
        # NOTE: this is inefficient because construct_domain() automatically
        # performs conversion to the target domain. It shouldn't do this.
        coeffs = sum((list(rep.values()) for rep in reps), [])
        opt.domain, _ = construct_domain(coeffs, opt=opt)

    _ring = PolynomialRing(opt.domain, opt.gens, opt.order)
    polys = list(map(_ring.from_dict, reps))

    if single:
        return _ring, polys[0]
    else:
        return _ring, polys


def _parse_symbols(symbols):
    if not symbols:
        raise GeneratorsNeeded("generators weren't specified")

    if isinstance(symbols, str):
        return _symbols(symbols, seq=True)
    elif isinstance(symbols, Expr):
        return symbols,
    elif is_sequence(symbols):
        if all(isinstance(s, str) for s in symbols):
            return _symbols(symbols)
        elif all(isinstance(s, Expr) for s in symbols):
            return symbols

    raise GeneratorsError('expected a string, Symbol or expression '
                          'or a non-empty sequence of strings, '
                          'Symbols or expressions')


_ring_cache = {}


class PolynomialRing(Ring, CompositeDomain, IPolys, _GCD):
    """A class for representing multivariate polynomial rings."""

    is_PolynomialRing = True

    has_assoc_Ring = True

    def __new__(cls, domain, symbols, order=lex):
        symbols = tuple(_parse_symbols(symbols))
        ngens = len(symbols)
        domain = DomainOpt.preprocess(domain)
        order = OrderOpt.preprocess(order)

        key = (cls.__name__, symbols, ngens, domain, order)
        obj = _ring_cache.get(key)

        if obj is None:
            if domain.is_Composite and set(symbols) & set(domain.symbols):
                raise GeneratorsError("polynomial ring and it's ground domain share generators")

            obj = object.__new__(cls)
            obj._hash = hash(key)
            obj.dtype = type('PolyElement', (PolyElement,), {'ring': obj})
            obj.symbols = symbols
            obj.ngens = ngens
            obj.domain = domain
            obj.order = order

            obj.zero_monom = Monomial((0,)*ngens)
            obj.gens = obj._gens()
            obj._gens_set = set(obj.gens)

            obj._one = [(obj.zero_monom, domain.one)]

            obj.leading_expv = lambda f: Monomial(max(f, key=order))

            obj.rep = str(domain) + '[' + ','.join(map(str, symbols)) + ']'

            for symbol, generator in zip(obj.symbols, obj.gens):
                if isinstance(symbol, Symbol):
                    name = symbol.name

                    if not hasattr(obj, name):
                        setattr(obj, name, generator)

            _ring_cache[key] = obj

        return obj

    def __getnewargs_ex__(self):
        return (self.domain, self.symbols), {'order': self.order}

    @property
    def characteristic(self):
        return self.domain.characteristic

    def _gens(self):
        """Return a list of polynomial generators."""
        one = self.domain.one
        _gens = []
        for i in range(self.ngens):
            expv = self.monomial_basis(i)
            poly = self.zero
            poly[expv] = one
            _gens.append(poly)
        return tuple(_gens)

    def __hash__(self):
        return self._hash

    def __eq__(self, other):
        return self is other

    def clone(self, symbols=None, domain=None, order=None):
        return self.__class__(domain or self.domain, symbols or self.symbols, order or self.order)

    def monomial_basis(self, i):
        """Return the ith-basis element."""
        basis = [0]*self.ngens
        basis[i] = 1
        return Monomial(basis)

    @property
    def zero(self):
        return self.dtype()

    @property
    def one(self):
        return self.dtype(self._one)

    def domain_new(self, element, orig_domain=None):
        return self.domain.convert(element, orig_domain)

    def ground_new(self, coeff):
        return self.term_new(self.zero_monom, coeff)

    def term_new(self, monom, coeff):
        coeff = self.domain_new(coeff)
        poly = self.zero
        if coeff:
            poly[monom] = coeff
        return poly

    def __call__(self, element):
        if isinstance(element, PolyElement):
            if self == element.ring:
                return element
            elif isinstance(self.domain, PolynomialRing) and self.domain.ring == element.ring:
                return self.ground_new(element)
            else:
                raise NotImplementedError
        elif isinstance(element, str):
            raise NotImplementedError
        elif isinstance(element, dict):
            return self.from_dict(element)
        elif isinstance(element, list):
            try:
                return self.from_terms(element)
            except ValueError:
                return self.from_list(element)
        elif isinstance(element, Expr):
            return self.convert(element)
        else:
            return self.ground_new(element)

    def from_dict(self, element):
        domain_new = self.domain_new
        poly = self.zero

        for monom, coeff in element.items():
            if isinstance(monom, int):
                monom = monom,
            coeff = domain_new(coeff)
            if coeff:
                poly[monom] = coeff

        return poly

    def from_terms(self, element):
        return self.from_dict(dict(element))

    def from_list(self, element):
        return self.from_dict(dmp_to_dict(element, self.ngens-1))

    def from_expr(self, expr):
        expr = sympify(expr)

        domain = self.domain
        mapping = dict(zip(self.symbols, self.gens))

        def _rebuild(expr):
            generator = mapping.get(expr)

            if generator is not None:
                return generator
            elif expr.is_Add:
                return functools.reduce(operator.add, map(_rebuild, expr.args))
            elif expr.is_Mul:
                return functools.reduce(operator.mul, map(_rebuild, expr.args))
            elif expr.is_Pow:
                c, a = expr.exp.as_coeff_Mul(rational=True)
                if c.is_Integer and c > 1:
                    return _rebuild(expr.base**a)**int(c)

            return domain.convert(expr)

        try:
            poly = _rebuild(expr)
        except CoercionFailed:
            raise ValueError('expected an expression convertible to a '
                             f'polynomial in {self}, got {expr}')
        else:
            return self(poly)

    def index(self, gen):
        """Compute index of ``gen`` in ``self.gens``."""
        if isinstance(gen, int):
            i = gen

            if 0 <= i and i < self.ngens:
                pass
            elif -self.ngens <= i and i <= -1:
                i = self.ngens + i
            else:
                raise ValueError(f'invalid generator index: {gen}')
        elif isinstance(gen, self.dtype):
            try:
                i = self.gens.index(gen)
            except ValueError:
                raise ValueError(f'invalid generator: {gen}')
        elif isinstance(gen, str):
            try:
                i = self.symbols.index(Symbol(gen))
            except ValueError:
                raise ValueError(f'invalid generator: {gen}')
        elif isinstance(gen, Expr):
            try:
                i = self.symbols.index(gen)
            except ValueError:
                raise ValueError(f'invalid generator: {gen}')
        else:
            raise ValueError(f'expected a polynomial generator, an integer, a string, an expression or None, got {gen}')

        return i

    def drop(self, *gens):
        """Remove specified generators from this ring."""
        indices = set(map(self.index, gens))
        symbols = [s for i, s in enumerate(self.symbols) if i not in indices]

        if not symbols:
            return self.domain
        else:
            return self.clone(symbols=symbols)

    def to_ground(self):
        if self.domain.is_Composite or self.domain.is_AlgebraicField:
            return self.clone(domain=self.domain.domain)
        else:
            raise ValueError(f'{self.domain} is not a composite or algebraic domain')

    @property
    def is_univariate(self):
        return len(self.gens) == 1

    @property
    def is_multivariate(self):
        return len(self.gens) > 1

    def eject(self, *gens):
        r"""
        Remove specified generators from the ring and inject them into
        its domain.

        """
        indices = set(map(self.index, gens))
        symbols = [s for i, s in enumerate(self.symbols) if i not in indices]
        gens = [gen for i, gen in enumerate(self.gens) if i not in indices]

        if not symbols:
            return self
        else:
            return self.clone(symbols=symbols, domain=self.drop(*gens))

    def to_expr(self, element):
        return element.as_expr()

    def _from_PythonIntegerRing(self, a, K0):
        return self(self.domain.convert(a, K0))

    def _from_PythonFiniteField(self, a, K0):
        if self.domain == K0:
            return self(a)
    _from_GMPYFiniteField = _from_PythonFiniteField

    def _from_PythonRationalField(self, a, K0):
        return self(self.domain.convert(a, K0))

    def _from_GMPYIntegerRing(self, a, K0):
        return self(self.domain.convert(a, K0))

    def _from_GMPYRationalField(self, a, K0):
        return self(self.domain.convert(a, K0))

    def _from_RealField(self, a, K0):
        return self(self.domain.convert(a, K0))

    def _from_ExpressionDomain(self, a, K0):
        if self.domain == K0:
            return self(a)

    def _from_AlgebraicField(self, a, K0):
        if self.domain == K0:
            return self(a)

    def _from_PolynomialRing(self, a, K0):
        try:
            return a.set_ring(self)
        except (CoercionFailed, GeneratorsError):
            return

    def _from_FractionField(self, a, K0):
        if self.domain == K0:
            return self.ground_new(a)

        (q,), r = a.numerator.div([a.denominator])

        if r.is_zero:
            return self.convert(q, K0.field.ring)

    @property
    def field(self):
        """Returns a field associated with ``self``."""
        return self.domain.frac_field(*self.symbols, order=self.order)

    def is_normal(self, a):
        return self.domain.is_normal(a.LC)

    def gcdex(self, a, b):
        """Extended GCD of ``a`` and ``b``."""
        return a.gcdex(b)

    def half_gcdex(self, a, b):
        """Half extended GCD of ``a`` and ``b``."""
        return a.half_gcdex(b)

    def gcd(self, a, b):
        """Returns GCD of ``a`` and ``b``."""
        return a.gcd(b)

    def lcm(self, a, b):
        """Returns LCM of ``a`` and ``b``."""
        return a.lcm(b)


class PolyElement(DomainElement, CantSympify, dict):
    """Element of multivariate distributed polynomial ring.

    See Also
    ========

    PolynomialRing

    """

    @property
    def parent(self):
        return self.ring

    _hash = None

    def __hash__(self):
        # XXX: This computes a hash of a dictionary, but currently we don't
        # protect dictionary from being changed so any use site modifications
        # will make hashing go wrong. Use this feature with caution until we
        # figure out how to make a safe API without compromising speed of this
        # low-level class.
        _hash = self._hash
        if _hash is None:
            self._hash = _hash = hash((self.ring, frozenset(self.items())))
        return _hash

    def __reduce__(self):
        return self.parent.__call__, (dict(self),)

    def copy(self):
        """Return a copy of polynomial self.

        Polynomials are mutable; if one is interested in preserving
        a polynomial, and one plans to use inplace operations, one
        can copy the polynomial. This method makes a shallow copy.

        Examples
        ========

        >>> R, x, y = ring('x, y', ZZ)
        >>> p = (x + y)**2
        >>> p1 = p.copy()
        >>> p2 = p
        >>> p[R.zero_monom] = 3
        >>> p
        x**2 + 2*x*y + y**2 + 3
        >>> p1
        x**2 + 2*x*y + y**2
        >>> p2
        x**2 + 2*x*y + y**2 + 3

        """
        return self.__class__(self)

    def set_ring(self, new_ring):
        if self.ring == new_ring:
            return self
        elif self.ring == new_ring.domain:
            return new_ring.ground_new(self)
        elif set(new_ring.symbols).issuperset(self.ring.symbols):
            terms = zip(*_dict_reorder(self, self.ring.symbols, new_ring.symbols))
            return new_ring.from_terms(terms)
        else:
            raise CoercionFailed(f"Can't set element ring to {new_ring}")

    def set_domain(self, new_domain):
        if self.ring.domain == new_domain:
            return self
        else:
            new_ring = self.ring.clone(domain=new_domain)
            return self.set_ring(new_ring)

    def as_expr(self, *symbols):
        if not symbols:
            symbols = self.ring.symbols
        elif len(symbols) != self.ring.ngens:
            raise ValueError(f'not enough symbols, expected {self.ring.ngens} got {len(symbols)}')

        to_expr = self.ring.domain.to_expr
        return expr_from_dict({monom: to_expr(self[monom]) for monom in self}, *symbols)

    def clear_denoms(self, convert=False):
        domain = self.ring.domain

        if not domain.is_Field:
            return domain.one, self

        if domain.has_assoc_Ring:
            ground_ring = domain.ring
        else:
            ground_ring = domain

        common = ground_ring.one
        lcm = ground_ring.lcm

        for coeff in self.values():
            common = lcm(common, coeff.denominator)

        f = self.mul_ground(domain.convert(common))

        if convert:
            f = f.set_domain(ground_ring)

        return common, f

    def _strip_zero(self):
        """Eliminate monomials with zero coefficient."""
        for k, v in list(self.items()):
            if not v:
                del self[k]

    def __setitem__(self, key, item):
        if not isinstance(key, Monomial):
            key = Monomial(key)
        super().__setitem__(key, item)

    def __eq__(self, other):
        """Equality test for polynomials.

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> p1 = (x + y)**2 + (x - y)**2
        >>> p1 == 4*x*y
        False
        >>> p1 == 2*(x**2 + y**2)
        True

        """
        if not other:
            return not self
        elif isinstance(other, self.ring.dtype):
            return dict.__eq__(self, other)
        elif isinstance(other, self.ring.field.dtype):
            return other.__eq__(self)
        elif len(self) > 1:
            return False
        else:
            return self.get(self.ring.zero_monom) == other

    def __ne__(self, other):
        return not self.__eq__(other)

    def almosteq(self, other, tolerance=None):
        """Approximate equality test for polynomials."""
        ring = self.ring

        if isinstance(other, ring.dtype):
            if set(self) != set(other):
                return False

            almosteq = ring.domain.almosteq

            for k in self:
                if not almosteq(self[k], other[k], tolerance):
                    return False
            return True
        elif len(self) > 1:
            return False
        else:
            try:
                other = ring.domain.convert(other)
            except CoercionFailed:
                return False
            else:
                return ring.domain.almosteq(self.coeff(1), other, tolerance)

    def sort_key(self):
        return len(self), self.terms()

    def drop(self, gen):
        ring = self.ring
        i = ring.index(gen)

        if ring.is_univariate:
            if self.is_ground:
                return self.coeff(1)
            else:
                raise ValueError(f"can't drop {gen}")
        else:
            symbols = list(ring.symbols)
            del symbols[i]
            ring = ring.clone(symbols=symbols)

            poly = ring.zero

            for k, v in self.items():
                if k[i] == 0:
                    K = list(k)
                    del K[i]
                    poly[K] = v
                else:
                    raise ValueError(f"can't drop {gen}")

            return poly

    def eject(self, *gens):
        ring = self.ring

        if not gens:
            return self

        if ring.is_univariate:
            raise ValueError("can't drop only generator to ground")

        indexes = [ring.index(gen) for gen in gens]
        ring = ring.eject(*indexes)

        poly = ring.zero
        gens = ring.domain.gens[0:len(indexes)]

        for monom, coeff in self.items():
            mon = tuple(monom[i] for i in range(self.ring.ngens) if i not in indexes)
            gc = functools.reduce(operator.mul, [x**n for x, n in zip(gens, (monom[i] for i in indexes))])
            if mon in poly:
                poly[mon] += gc.mul_ground(coeff)
            else:
                poly[mon] = gc.mul_ground(coeff)

        return poly

    def inject(self, front=False):
        ring = self.ring
        domain = ring.domain

        if not (domain.is_Composite or domain.is_AlgebraicField):
            return self

        new_ring = ring.to_ground()
        new_ring = new_ring.inject(*domain.symbols, front=front)

        poly = new_ring.zero

        for monom, coeff in self.items():
            coeff = coeff.to_dict()
            for cmonom, ccoeff in coeff.items():
                if front:
                    cmonom += monom
                else:
                    cmonom = monom + cmonom

                poly[cmonom] = ccoeff

        return poly

    def to_dense(self):
        return dmp_from_dict(self, self.ring.ngens-1, self.ring.domain)

    def to_dict(self):
        return dict(self)

    def str(self, printer, precedence, exp_pattern, mul_symbol):
        if not self:
            return printer._print(self.ring.domain.zero)
        prec_add = precedence['Add']
        prec_atom = precedence['Atom']
        ring = self.ring
        symbols = ring.symbols
        ngens = ring.ngens
        zm = ring.zero_monom
        sexpvs = []
        for expv, coeff in self.terms():
            normal = ring.domain.is_normal(coeff)
            sign = ' + ' if normal else ' - '
            sexpvs.append(sign)
            if expv == zm:
                scoeff = printer._print(coeff)
                if scoeff.startswith('-'):
                    scoeff = scoeff[1:]
            else:
                if not normal:
                    coeff = -coeff
                if coeff != 1:
                    scoeff = printer.parenthesize(coeff, prec_add)
                else:
                    scoeff = ''
            sexpv = []
            for i in range(ngens):
                exp = expv[i]
                if not exp:
                    continue
                symbol = printer.parenthesize(symbols[i], prec_atom-1)
                if exp != 1:
                    sexpv.append(exp_pattern % (symbol, exp))
                else:
                    sexpv.append(f'{symbol}')
            if scoeff:
                sexpv = [scoeff] + sexpv
            sexpvs.append(mul_symbol.join(sexpv))
        head = sexpvs.pop(0)
        if head == ' - ':
            sexpvs.insert(0, '-')
        return ''.join(sexpvs)

    @property
    def is_generator(self):
        return self in self.ring._gens_set

    @property
    def is_ground(self):
        return not self or (len(self) == 1 and self.ring.zero_monom in self)

    @property
    def is_monomial(self):
        return not self or (len(self) == 1 and self.LC == 1)

    @property
    def is_term(self):
        return len(self) <= 1

    @property
    def is_zero(self):
        return not self

    @property
    def is_one(self):
        return self == self.ring.one

    @property
    def is_monic(self):
        return self.LC == self.ring.domain.one

    @property
    def is_primitive(self):
        return self.content() == self.ring.domain.one

    @property
    def is_linear(self):
        return all(sum(monom) <= 1 for monom in self)

    @property
    def is_quadratic(self):
        return all(sum(monom) <= 2 for monom in self)

    @property
    def is_irreducible(self):
        _, factors = self.factor_list()
        if not factors:
            return True
        elif len(factors) > 1:
            return False
        else:
            return factors[0][1] == 1

    @property
    def is_homogeneous(self):
        if self.is_zero:
            return True

        tdeg = sum(self.LM)

        for monom in self:
            _tdeg = sum(monom)

            if _tdeg != tdeg:
                return False

        return True

    def __neg__(self):
        return self.__class__({monom: -self[monom] for monom in self})

    def __pos__(self):
        return self

    def __abs__(self):
        return self.__class__({monom: abs(self[monom]) for monom in self})

    def __add__(self, other):
        """Add two polynomials.

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> (x + y)**2 + (x - y)**2
        2*x**2 + 2*y**2

        """
        ring = self.ring
        if not isinstance(other, ring.dtype):
            try:
                other = ring.convert(other)
            except CoercionFailed:
                return NotImplemented
        p = self.copy()
        if not other:
            return p
        get = p.get
        zero = ring.domain.zero
        for k, v in other.items():
            p[k] = get(k, zero) + v
        p._strip_zero()
        return p

    def __radd__(self, other):
        return self.__add__(other)

    def __sub__(self, other):
        """Subtract polynomial other from self.

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> p1 = x + y**2
        >>> p2 = x*y + y**2
        >>> p1 - p2
        -x*y + x

        """
        ring = self.ring
        if not isinstance(other, ring.dtype):
            try:
                other = ring.convert(other)
            except CoercionFailed:
                return NotImplemented
        p = self.copy()
        if not other:
            return p
        get = p.get
        zero = ring.domain.zero
        for k, v in other.items():
            p[k] = get(k, zero) - v
        p._strip_zero()
        return p

    def __rsub__(self, other):
        """Substract self from other, with other convertible to the coefficient domain.

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> p = x + y
        >>> 4 - p
        -x - y + 4

        """
        return (-self).__add__(other)

    def __mul__(self, other):
        """Multiply two polynomials.

        Examples
        ========

        >>> _, x, y = ring('x, y', QQ)
        >>> p1 = x + y
        >>> p2 = x - y
        >>> p1*p2
        x**2 - y**2

        """
        ring = self.ring
        if not isinstance(other, ring.dtype):
            try:
                other = ring.convert(other)
            except CoercionFailed:
                return NotImplemented
        p = ring.zero
        if not self or not other:
            return p
        get = p.get
        zero = ring.domain.zero
        for exp1, v1 in self.items():
            for exp2, v2 in other.items():
                exp = exp1*exp2
                p[exp] = get(exp, zero) + v1*v2
        p._strip_zero()
        return p

    def __rmul__(self, other):
        """Multiply other to self with other in the coefficient domain of self.

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> p = x + y
        >>> 4 * p
        4*x + 4*y

        """
        return self.__mul__(other)

    def __pow__(self, n):
        """Raise polynomial to power `n`.

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> p = x + y**2
        >>> p**3
        x**3 + 3*x**2*y**2 + 3*x*y**4 + y**6

        """
        ring = self.ring
        n = int(n)

        if n < 0:
            raise ValueError('negative exponent')
        elif not n:
            return ring.one
        elif len(self) == 1:
            monom, coeff = list(self.items())[0]
            p = ring.zero
            p[monom**n] = coeff**n
            return p
        elif n == 1:
            return self.copy()
        elif n == 2:
            return self._square()
        elif n == 3:
            return self*self._square()
        elif len(self) <= 5:
            return self._pow_multinomial(n)
        else:
            return self._pow_generic(n)

    def _pow_generic(self, n):
        p = self.ring.one
        c = self

        while n:
            if n & 1:
                p *= c
                n -= 1

            c = c._square()
            n //= 2

        return p

    def _pow_multinomial(self, n):
        multinomials = multinomial_coefficients(len(self), n).items()
        zero_monom = self.ring.zero_monom
        terms = self.items()
        zero = self.ring.domain.zero
        poly = self.ring.zero

        for multinomial, multinomial_coeff in multinomials:
            product_monom = zero_monom
            product_coeff = multinomial_coeff

            for exp, (monom, coeff) in zip(multinomial, terms):
                if exp:
                    product_monom *= monom**exp
                    product_coeff *= coeff**exp

            monom = product_monom
            coeff = poly.get(monom, zero) + product_coeff

            if coeff:
                poly[monom] = coeff
            elif monom in poly:
                del poly[monom]

        return poly

    def _square(self):
        """Square of a polynomial.

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> p = x + y**2
        >>> p._square()
        x**2 + 2*x*y**2 + y**4

        """
        ring = self.ring
        p = ring.zero
        get = p.get
        keys = list(self)
        zero = ring.domain.zero
        for i in range(len(keys)):
            k1 = keys[i]
            pk = self[k1]
            for j in range(i):
                k2 = keys[j]
                exp = k1*k2
                p[exp] = get(exp, zero) + pk*self[k2]
        p += p
        get = p.get
        for k, v in self.items():
            k2 = k**2
            p[k2] = get(k2, zero) + v**2
        p._strip_zero()
        return p

    def __divmod__(self, other):
        ring = self.ring

        if not other:
            raise ZeroDivisionError('polynomial division')
        elif isinstance(other, ring.dtype):
            (q,), r = self.div([other])
            return q, r
        elif isinstance(other, PolyElement):
            if isinstance(ring.domain, PolynomialRing) and ring.domain.ring == other.ring:
                pass
            else:
                return NotImplemented

        try:
            other = ring.domain_new(other)
        except CoercionFailed:
            return NotImplemented
        else:
            return self.quo_ground(other), self.trunc_ground(other)

    def __mod__(self, other):
        return divmod(self, other)[1]

    def __floordiv__(self, other):
        return divmod(self, other)[0]

    def __truediv__(self, other):
        ring = self.ring

        if not other:
            raise ZeroDivisionError('polynomial division')
        elif isinstance(other, ring.domain.dtype):
            return self.quo_ground(other)

        try:
            other = ring.domain_new(other)
        except CoercionFailed:
            return NotImplemented
        else:
            return self.quo_ground(other)

    def div(self, fv):
        """Division algorithm, see :cite:`Cox2015ideals`, p. 64.

        fv array of polynomials
           return qv, r such that
           self = sum(fv[i]*qv[i]) + r

        All polynomials are required not to be Laurent polynomials.

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> f = x**3
        >>> f0 = x - y**2
        >>> f1 = x - y
        >>> qv, r = f.div((f0, f1))
        >>> qv[0]
        x**2 + x*y**2 + y**4
        >>> qv[1]
        0
        >>> r
        y**6

        """
        ring = self.ring
        order = ring.order
        if any(not f for f in fv):
            raise ZeroDivisionError('polynomial division')
        if any(f.ring != ring for f in fv):
            raise ValueError('self and f must have the same ring')
        if not self:
            return [ring.zero], ring.zero
        s = len(fv)
        qv = [ring.zero for i in range(s)]
        p = self.copy()
        r = ring.zero
        expvs = [fx.leading_expv() for fx in fv]
        while p:
            i = 0
            divoccurred = 0
            while i < s and divoccurred == 0:
                lt = p.leading_term()
                term = lt.quo_term((expvs[i], fv[i][expvs[i]]))
                if term:
                    expv1, c = term.LT
                    qv[i] = qv[i]._iadd_monom((expv1, c))
                    p = p._iadd_poly_monom(fv[i], (expv1, -c))
                    divoccurred = 1
                    if p and order(p.LM) >= order(lt.LM):
                        raise PolynomialDivisionFailed(self, fv[i], self.ring)
                else:
                    i += 1
            if not divoccurred:
                expv = p.leading_expv()
                r = r._iadd_monom((expv, p[expv]))
                del p[expv]
        r._hash = None
        return qv, r

    def exquo(self, other):
        q, r = divmod(self, other)

        if not r:
            return q
        else:
            raise ExactQuotientFailed(self, other)

    def _iadd_monom(self, mc):
        """Add to self the monomial coeff*x0**i0*x1**i1*...
        unless self is a generator -- then just return the sum of the two.

        mc is a tuple, (monom, coeff), where monomial is (i0, i1, ...)

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> p = x**4 + 2*y
        >>> m = (1, 2)
        >>> p1 = p._iadd_monom((m, 5))
        >>> p1
        x**4 + 5*x*y**2 + 2*y
        >>> p1 is p
        True
        >>> p = x
        >>> p1 = p._iadd_monom((m, 5))
        >>> p1
        5*x*y**2 + x
        >>> p1 is p
        False

        """
        if self in self.ring._gens_set:
            cpself = self.copy()
        else:
            cpself = self
        expv, coeff = mc
        c = cpself.get(expv)
        if c is None:
            cpself[expv] = coeff
        else:
            c += coeff
            if c:
                cpself[expv] = c
            else:
                del cpself[expv]
        return cpself

    def _iadd_poly_monom(self, p2, mc):
        """Add to self the product of (p)*(coeff*x0**i0*x1**i1*...)
        unless self is a generator -- then just return the sum of the two.

        mc is a tuple, (monom, coeff), where monomial is (i0, i1, ...)

        Examples
        ========

        >>> _, x, y, z = ring('x, y, z', ZZ)
        >>> p1 = x**4 + 2*y
        >>> p2 = y + z
        >>> m = (1, 2, 3)
        >>> p1 = p1._iadd_poly_monom(p2, (m, 3))
        >>> p1
        x**4 + 3*x*y**3*z**3 + 3*x*y**2*z**4 + 2*y

        """
        p1 = self
        if p1 in p1.ring._gens_set:
            p1 = p1.copy()
        m, c = mc
        get = p1.get
        zero = p1.ring.domain.zero
        for k, v in p2.items():
            ka = k*m
            coeff = get(ka, zero) + v*c
            if coeff:
                p1[ka] = coeff
            else:
                del p1[ka]
        return p1

    def degree(self, x=0):
        """
        The leading degree in ``x`` or the main variable.

        Note that the degree of 0 is negative infinity (the Diofant object -oo).

        """
        i = self.ring.index(x)
        return max((monom[i] for monom in self), default=-oo)

    def degree_list(self):
        """
        A tuple containing leading degrees in all variables.

        Note that the degree of 0 is negative infinity (the Diofant object -oo)

        """
        return tuple(self.degree(i) for i in range(self.ring.ngens))

    def tail_degree(self, x=0):
        """
        The tail degree in ``x`` or the main variable.

        Note that the degree of 0 is negative infinity (the Diofant object -oo)

        """
        i = self.ring.index(x)
        return min((monom[i] for monom in self), default=-oo)

    def tail_degrees(self):
        """
        A tuple containing tail degrees in all variables.

        Note that the degree of 0 is negative infinity (the Diofant object -oo)

        """
        return tuple(self.tail_degree(i) for i in range(self.ring.ngens))

    def total_degree(self):
        """Returns the total degree."""
        return max(sum(m) for m in self.monoms())

    def leading_expv(self):
        """Leading monomial tuple according to the monomial ordering.

        Examples
        ========

        >>> _, x, y, z = ring('x, y, z', ZZ)
        >>> p = x**4 + x**3*y + x**2*z**2 + z**7
        >>> p.leading_expv()
        (4, 0, 0)

        """
        if self:
            return self.ring.leading_expv(self)

    def _get_coeff(self, expv):
        return self.get(expv, self.ring.domain.zero)

    def coeff(self, element):
        """
        Returns the coefficient that stands next to the given monomial.

        Parameters
        ==========

        element : PolyElement (with ``is_monomial = True``) or 1

        Examples
        ========

        >>> _, x, y, z = ring('x y z', ZZ)
        >>> f = 3*x**2*y - x*y*z + 7*z**3 + 23

        >>> f.coeff(x**2*y)
        3
        >>> f.coeff(x*y)
        0
        >>> f.coeff(1)
        23

        """
        if element == 1:
            return self._get_coeff(self.ring.zero_monom)
        elif isinstance(element, self.ring.dtype) and element.is_monomial:
            monom = element.monoms().pop()
            return self._get_coeff(monom)
        elif is_sequence(element) and all(isinstance(n, int) for n in element):
            return self._get_coeff(element)

        raise ValueError(f'expected a monomial, got {element}')

    @property
    def LC(self):
        return self._get_coeff(self.leading_expv())

    @property
    def LM(self):
        expv = self.leading_expv()
        if expv is None:
            return self.ring.zero_monom
        else:
            return expv

    def leading_monom(self):
        """
        Leading monomial as a polynomial element.

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> (3*x*y + y**2).leading_monom()
        x*y

        """
        p = self.ring.zero
        expv = self.leading_expv()
        if expv:
            p[expv] = self.ring.domain.one
        return p

    @property
    def LT(self):
        expv = self.leading_expv()
        if expv is None:
            return self.ring.zero_monom, self.ring.domain.zero
        else:
            return expv, self._get_coeff(expv)

    def leading_term(self):
        """Leading term as a polynomial element.

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ)
        >>> (3*x*y + y**2).leading_term()
        3*x*y

        """
        p = self.ring.zero
        expv = self.leading_expv()
        if expv:
            p[expv] = self[expv]
        return p

    def _sorted(self, seq, order):
        if order is None:
            order = self.ring.order
        else:
            order = OrderOpt.preprocess(order)

        return sorted(seq, key=lambda monom: order(monom[0]), reverse=True)

    def coeffs(self, order=None):
        """Ordered list of polynomial coefficients.

        Parameters
        ==========

        order : :class:`~diofant.polys.polyoptions.Order` or coercible, optional

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ, lex)
        >>> f = x*y**7 + 2*x**2*y**3

        >>> f.coeffs()
        [2, 1]
        >>> f.coeffs(grlex)
        [1, 2]

        """
        return [coeff for _, coeff in self.terms(order)]

    def all_coeffs(self):
        if self.ring.is_univariate:
            if self.is_zero:
                return [self.parent.domain.zero]
            else:
                return self.to_dense()
        else:
            raise PolynomialError('multivariate polynomials not supported')

    def monoms(self, order=None):
        """Ordered list of polynomial monomials.

        Parameters
        ==========

        order : :class:`~diofant.polys.polyoptions.Order` or coercible, optional

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ, lex)
        >>> f = x*y**7 + 2*x**2*y**3

        >>> f.monoms()
        [(2, 3), (1, 7)]
        >>> f.monoms(grlex)
        [(1, 7), (2, 3)]

        """
        return [monom for monom, _ in self.terms(order)]

    def terms(self, order=None):
        """Ordered list of polynomial terms.

        Parameters
        ==========

        order : :class:`~diofant.polys.polyoptions.Order` or coercible, optional

        Examples
        ========

        >>> _, x, y = ring('x, y', ZZ, lex)
        >>> f = x*y**7 + 2*x**2*y**3

        >>> f.terms()
        [((2, 3), 2), ((1, 7), 1)]
        >>> f.terms(grlex)
        [((1, 7), 1), ((2, 3), 2)]

        """
        return self._sorted(self.items(), order)

    def content(self):
        """Returns GCD of polynomial's coefficients."""
        ring = self.ring
        domain = ring.domain
        cont = domain.zero
        gcd = domain.gcd

        for coeff in self.values():
            cont = gcd(cont, coeff)

            if cont == domain.one:
                break

        if not ring.is_normal(self):
            cont = -cont

        return cont

    def primitive(self):
        """Returns content and a primitive polynomial."""
        cont = self.content()
        prim = self.copy()
        if not prim.is_zero:
            prim = prim.quo_ground(cont)
        return cont, prim

    def monic(self):
        """Divides all coefficients by the leading coefficient."""
        if not self:
            return self
        else:
            return self.exquo_ground(self.LC)

    def mul_ground(self, x):
        if not x:
            return self.ring.zero

        return self.__class__({monom: self[monom]*x for monom in self})

    def mul_monom(self, monom):
        terms = {f_monom*monom: self[f_monom] for f_monom in self}
        return self.__class__(terms)

    def mul_term(self, term):
        monom, coeff = term

        if not self or not coeff:
            return self.ring.zero
        elif monom == self.ring.zero_monom:
            return self.mul_ground(coeff)

        terms = {f_monom*monom: self[f_monom]*coeff for f_monom in self}
        return self.__class__(terms)

    def quo_ground(self, x):
        domain = self.ring.domain

        if not x:
            raise ZeroDivisionError('polynomial division')
        if not self or x == domain.one:
            return self

        if domain.is_Field:
            quo = domain.quo
            terms = {monom: quo(self[monom], x) for monom in self}
        else:
            terms = {monom: self[monom]//x for monom in self}

        p = self.__class__(terms)
        p._strip_zero()
        return p

    def exquo_ground(self, x):
        domain = self.ring.domain

        if not x:
            raise ZeroDivisionError('polynomial division')
        if not self or x == domain.one:
            return self

        terms = {monom: domain.exquo(self[monom], x) for monom in self}

        p = self.__class__(terms)
        p._strip_zero()
        return p

    def quo_term(self, term):
        monom, coeff = term

        if not coeff:
            raise ZeroDivisionError('polynomial division')
        elif not self:
            return self.ring.zero

        ring = self.ring
        domain = ring.domain
        p = ring.zero

        for tm, tc in self.items():
            if monom != self.ring.zero_monom:
                tm /= monom
            if any(_ < 0 for _ in tm):
                continue
            if domain.is_Field or not tc % coeff:
                p[tm] = domain.quo(tc, coeff)

        return p

    def trunc_ground(self, p):
        if self.ring.domain.is_IntegerRing:
            terms = {}

            for monom, coeff in self.items():
                coeff = coeff % p
                terms[monom] = symmetric_residue(coeff, p)
        else:
            terms = {monom: self[monom] % p for monom in self}

        poly = self.__class__(terms)
        poly._strip_zero()
        return poly

    def extract_ground(self, g):
        f = self
        fc = f.content()
        gc = g.content()

        gcd = f.ring.domain.gcd(fc, gc)

        f = f.quo_ground(gcd)
        g = g.quo_ground(gcd)

        return gcd, f, g

    def _norm(self, norm_func):
        if not self:
            return self.ring.domain.zero
        else:
            return norm_func([abs(coeff) for coeff in self.values()])

    def max_norm(self):
        return self._norm(max)

    def l1_norm(self):
        return self._norm(sum)

    def deflate(self, *G):
        ring = self.ring
        polys = [self] + list(G)

        J = [0]*ring.ngens

        for p in polys:
            for monom in p:
                for i, m in enumerate(monom):
                    J[i] = math.gcd(J[i], m)

        for i, b in enumerate(J):
            if not b:
                J[i] = 1

        J = tuple(J)

        if all(b == 1 for b in J):
            return J, polys

        H = []

        for p in polys:
            h = ring.zero

            for I, coeff in p.items():
                N = [i//j for i, j in zip(I, J)]
                h[N] = coeff

            H.append(h)

        return J, H

    def inflate(self, J):
        poly = self.ring.zero

        for I, coeff in self.items():
            N = [i*j for i, j in zip(I, J)]
            poly[N] = coeff

        return poly

    def lcm(self, g):
        f = self
        domain = f.ring.domain

        if not domain.is_Field:
            fc, f = f.primitive()
            gc, g = g.primitive()
            c = domain.lcm(fc, gc)

        h = (f*g)//f.gcd(g)

        if not domain.is_Field:
            return h.mul_ground(c)
        else:
            return h.monic()

    def gcd(self, other):
        return self.cofactors(other)[0]

    def cofactors(self, other):
        if not self and not other:
            zero = self.ring.zero
            return zero, zero, zero
        elif not self:
            h, cff, cfg = self._gcd_zero(other)
            return h, cff, cfg
        elif not other:
            h, cfg, cff = other._gcd_zero(self)
            return h, cff, cfg

        J, (f, g) = self.deflate(other)
        h, cff, cfg = f._gcd(g)

        return h.inflate(J), cff.inflate(J), cfg.inflate(J)

    def _gcd_zero(self, other):
        one, zero = self.ring.one, self.ring.zero
        if self.ring.domain.is_Field:
            return other.monic(), zero, self.ring.ground_new(other.LC)
        else:
            if not self.ring.is_normal(other):
                return -other, zero, -one
            else:
                return other, zero, one

    def _gcd(self, other):
        ring = self.ring

        if ring.domain.is_RationalField:
            return self._gcd_QQ(other)
        elif ring.domain.is_IntegerRing:
            return self._gcd_ZZ(other)
        elif ring.domain.is_AlgebraicField:
            return self._gcd_AA(other)
        elif not ring.domain.is_Exact:
            try:
                exact = ring.domain.get_exact()
            except DomainError:
                return ring.one, self, other

            f, g = map(lambda x: x.set_domain(exact), (self, other))

            return tuple(map(lambda x: x.set_domain(ring.domain), f.cofactors(g)))
        elif ring.domain.is_Field:
            return self.ring._ff_prs_gcd(self, other)
        else:
            return self.ring._rr_prs_gcd(self, other)

    def _gcd_ZZ(self, other):
        if query('USE_HEU_GCD'):
            try:
                return heugcd(self, other)
            except HeuristicGCDFailed:  # pragma: no cover
                pass

        _gcd_zz_methods = {'modgcd': modgcd,
                           'prs': self.ring._rr_prs_gcd}

        method = _gcd_zz_methods[query('FALLBACK_GCD_ZZ_METHOD')]
        return method(self, other)

    def _gcd_QQ(self, g):
        f = self
        ring = f.ring

        cf, f = f.clear_denoms(convert=True)
        cg, g = g.clear_denoms(convert=True)

        h, cff, cfg = map(lambda _: _.set_ring(ring), f._gcd_ZZ(g))

        c, h = h.LC, h.monic()

        cff = cff.mul_ground(ring.domain.quo(c, cf))
        cfg = cfg.mul_ground(ring.domain.quo(c, cg))

        return h, cff, cfg

    def _gcd_AA(self, g):
        _gcd_aa_methods = {'modgcd': func_field_modgcd,
                           'prs': self.ring._ff_prs_gcd}

        method = _gcd_aa_methods[query('GCD_AA_METHOD')]
        return method(self, g)

    def terms_gcd(self):
        if self.is_zero:
            return (0,)*self.ring.ngens, self

        G = functools.reduce(Monomial.gcd, self)

        if all(g == 0 for g in G):
            return G, self

        f = self.ring.zero

        for monom, coeff in self.items():
            f[monom/G] = coeff

        return G, f

    def cancel(self, g, include=True):
        """
        Cancel common factors in a rational function ``f/g``.

        Examples
        ========

        >>> R, x, y = ring('x y', ZZ)

        >>> (2*x**2 - 2).cancel(x**2 - 2*x + 1)
        (2*x + 2, x - 1)

        """
        f = self
        ring = f.ring
        domain = ring.domain

        if not (domain.is_Field and domain.has_assoc_Ring):
            _, p, q = f.cofactors(g)
            cp, cq = domain.one, domain.one
        else:
            new_ring = ring.clone(domain=domain.ring)

            cq, f = f.clear_denoms(convert=True)
            cp, g = g.clear_denoms(convert=True)

            _, p, q = f.cofactors(g)
            _, cp, cq = new_ring.domain.cofactors(cp, cq)

            p = p.set_ring(ring)
            q = q.set_ring(ring)

        p_neg = not ring.is_normal(p)
        q_neg = not ring.is_normal(q)

        if p_neg and q_neg:
            p, q = -p, -q
        elif p_neg:
            cp, p = -cp, -p
        elif q_neg:
            cp, q = -cp, -q

        if not include:
            return cp, cq, p, q

        p = p.mul_ground(cp)
        q = q.mul_ground(cq)

        return p, q

    def diff(self, x=0, m=1):
        """Computes partial derivative in ``x``.

        Examples
        ========

        >>> _, x, y = ring('x y', ZZ)
        >>> p = x + x**2*y**3
        >>> p.diff(x)
        2*x*y**3 + 1

        """
        ring = self.ring
        i = ring.index(x)
        x = ring.monomial_basis(i)
        x = x**m
        g = ring.zero if m else self.compose(ring.gens[i], ring.zero)
        for expv, coeff in self.items():
            if expv[i]:
                e = expv/x
                for j in range(expv[i], expv[i] - m, -1):
                    coeff *= j
                g[e] = coeff
        g._strip_zero()
        return g

    def integrate(self, x=0, m=1):
        """Computes indefinite integral in ``x``."""
        ring = self.ring
        i = ring.index(x)
        x = ring.monomial_basis(i)
        x = x**m
        g = ring.zero
        for expv, coeff in self.items():
            e = expv*x
            for j in range(expv[i] + m, expv[i], -1):
                coeff /= j
            g[e] = coeff
        g._strip_zero()
        return g

    def __call__(self, *values):
        if 0 < len(values) <= self.ring.ngens:
            return self.eval(list(zip(self.ring.gens, values)))
        else:
            raise ValueError(f'expected at least 1 and at most {self.ring.ngens} values, got {len(values)}')

    def eval(self, x=0, a=0):
        f = self

        if isinstance(x, list) and not a:
            (X, a), x = x[0], x[1:]
            f = f.eval(X, a)

            if not x:
                return f
            else:
                x = [(Y.drop(X), a) for (Y, a) in x]
                return f.eval(x)

        ring = f.ring
        i = ring.index(x)
        a = ring.domain.convert(a)

        if ring.is_univariate:
            result = ring.domain.zero

            for (n,), coeff in f.items():
                result += coeff*a**n

            return result
        else:
            poly = ring.drop(x).zero

            for monom, coeff in f.items():
                n, monom = monom[i], monom[:i] + monom[i+1:]
                coeff = coeff*a**n

                if monom in poly:
                    coeff += poly[monom]

                    if coeff:
                        poly[monom] = coeff
                    else:
                        del poly[monom]
                else:
                    if coeff:
                        poly[monom] = coeff

            return poly

    def compose(self, x, a=None):
        """Computes the functional composition."""
        ring = self.ring
        poly = ring.zero

        if a is not None:
            replacements = [(x, a)]
        else:
            if isinstance(x, list):
                replacements = list(x)
            elif isinstance(x, dict):
                replacements = sorted(x.items(), key=lambda k: ring.index(k[0]))
            else:
                raise ValueError('expected a generator, value pair a sequence of such pairs')

        for monom, coeff in self.items():
            monom = list(monom)
            subpoly = ring.one

            for x, g in replacements:
                i, g = ring.index(x), ring(g)
                n, monom[i] = monom[i], 0
                if n:
                    subpoly *= g**n

            subpoly = subpoly.mul_term((monom, coeff))
            poly += subpoly

        return poly

    def discriminant(self):
        """Computes discriminant of a polynomial."""
        ring = self.ring

        d = self.degree()

        if d <= 0:
            return ring.zero.drop(0)
        else:
            s = (-1)**((d*(d - 1)) // 2)
            c = self.eject(*ring.gens[1:]).LC

            return self.resultant(self.diff()) // (c*s)

    def shift(self, a):
        if self.ring.is_univariate:
            return self.compose(0, self.ring.gens[0] + a)
        else:
            raise MultivariatePolynomialError('polynomial shift')

    def slice(self, m, n, x=0):
        ring = self.ring
        poly = ring.zero
        j = ring.index(x)

        for monom, coeff in self.items():
            if not n > monom[j] >= m:
                if ring.is_univariate:
                    continue
                else:
                    monom = monom[:j] + (0,) + monom[j + 1:]

            if monom in poly:
                poly[monom] += coeff
            else:
                poly[monom] = coeff

        return poly

    def prem(self, other):
        """Polynomial pseudo-remainder.

        Examples
        ========

        >>> R, x, y = ring('x y', ZZ)

        >>> (x**2 + x*y).prem(2*x + 2)
        -4*y + 4

        References
        ==========

        * :cite:`Knuth1985seminumerical`, p. 407.

        """
        ring = self.ring

        if not isinstance(other, ring.dtype):
            other = ring.convert(other)

        f, g = self, other

        if ring.is_multivariate:
            f, g = map(lambda _: _.eject(*ring.gens[1:]), (f, g))
            r = f.prem(g)
            return r.inject()

        ring = f.ring

        df = f.degree()
        dg = g.degree()

        if dg < 0:
            raise ZeroDivisionError('polynomial division')

        r, dr = f, df

        if df < dg:
            return r

        x = ring.gens[0]
        n = df - dg + 1
        lc_g = g.LC

        while True:
            lc_r = r.LC
            n -= 1

            r *= lc_g
            r -= g*x**(dr - dg)*lc_r
            dr = r.degree()

            if dr < dg:
                break

        r *= lc_g**n

        return r

    def resultant(self, other, includePRS=False):
        """
        Computes resultant of two polynomials in `K[X]`.

        Examples
        ========

        >>> R, x, y = ring('x y', ZZ)

        >>> f = 3*x**2*y - y**3 - 4
        >>> g = x**2 + x*y**3 - 9

        >>> f.resultant(g)
        -3*y**10 - 12*y**7 + y**6 - 54*y**4 + 8*y**3 + 729*y**2 - 216*y + 16

        """
        ring = self.ring
        domain = ring.domain

        if (not includePRS and query('USE_COLLINS_RESULTANT') and
                (domain.is_IntegerRing or domain.is_RationalField)):
            return ring._collins_resultant(self, other)

        res = ring._primitive_prs(self, other)

        return res if includePRS else res[0]

    def subresultants(self, other):
        """
        Computes subresultant PRS of two polynomials in `K[X]`.

        Examples
        ========

        >>> R, x, y = ring('x y', ZZ)

        >>> f = 3*x**2*y - y**3 - 4
        >>> g = x**2 + x*y**3 - 9

        >>> a = 3*x*y**4 + y**3 - 27*y + 4
        >>> b = -3*y**10 - 12*y**7 + y**6 - 54*y**4 + 8*y**3 + 729*y**2 - 216*y + 16

        >>> f.subresultants(g) == [f, g, a, b]
        True

        """
        return self.resultant(other, includePRS=True)[1]

    def half_gcdex(self, other):
        """
        Half extended Euclidean algorithm in `F[x]`.

        Returns ``(s, h)`` such that ``h = gcd(self, other)``
        and ``s*self = h (mod other)``.

        Examples
        ========

        >>> R, x = ring('x', QQ)

        >>> f = x**4 - 2*x**3 - 6*x**2 + 12*x + 15
        >>> g = x**3 + x**2 - 4*x - 4

        >>> f.half_gcdex(g)
        (-1/5*x + 3/5, x + 1)

        """
        ring = self.ring
        if ring.is_multivariate:
            raise MultivariatePolynomialError('half extended Euclidean algorithm')

        domain = ring.domain

        if not domain.is_Field:
            raise DomainError(f"can't compute half extended GCD over {domain}")

        a, b = ring.one, ring.zero
        f, g = self, other

        while g:
            q, r = divmod(f, g)
            f, g = g, r
            a, b = b, a - q*b

        a = a.quo_ground(f.LC)
        f = f.monic()

        return a, f

    def gcdex(self, other):
        """
        Extended Euclidean algorithm in `F[x]`.

        Returns ``(s, t, h)`` such that ``h = gcd(self, other)`` and
        ``s*self + t*other = h``.

        Examples
        ========

        >>> R, x = ring('x', QQ)

        >>> f = x**4 - 2*x**3 - 6*x**2 + 12*x + 15
        >>> g = x**3 + x**2 - 4*x - 4

        >>> f.gcdex(g)
        (-1/5*x + 3/5, 1/5*x**2 - 6/5*x + 2, x + 1)

        """
        s, h = self.half_gcdex(other)
        t = h - self*s
        t //= other
        return s, t, h

    # The following methods aren't ported (yet) to polynomial
    # representation independent algorithm implementations.

    def decompose(self):
        if self.ring.is_univariate:
            return self.ring.dup_decompose(self)
        else:
            raise MultivariatePolynomialError('polynomial decomposition')

    def sturm(self):
        if self.ring.is_univariate:
            return self.ring.dup_sturm(self)
        else:
            raise MultivariatePolynomialError('sturm sequence')

    @property
    def is_cyclotomic(self):
        if self.ring.is_univariate:
            return self.ring.dup_cyclotomic_p(self)
        else:
            raise AttributeError('cyclotomic polynomial')

    @property
    def is_squarefree(self):
        return self.ring.dmp_sqf_p(self)

    def sqf_norm(self):
        return self.ring.dmp_sqf_norm(self)

    def sqf_part(self):
        return self.ring.dmp_sqf_part(self)

    def sqf_list(self):
        return self.ring.dmp_sqf_list(self)

    def factor_list(self):
        return self.ring.dmp_factor_list(self)



"""Tests for tools for manipulation of expressions using paths."""

import pytest

from diofant import E, cos, sin
from diofant.abc import t, x, y, z
from diofant.simplify.epathtools import EPath, epath


__all__ = ()


def test_epath_select():
    expr = [((x, 1, t), 2), ((3, y, 4), z)]

    assert epath('/*', expr) == [((x, 1, t), 2), ((3, y, 4), z)]
    assert epath('/*/*', expr) == [(x, 1, t), 2, (3, y, 4), z]
    assert epath('/*/*/*', expr) == [x, 1, t, 3, y, 4]
    assert epath('/*/*/*/*', expr) == []

    assert epath('/[:]', expr) == [((x, 1, t), 2), ((3, y, 4), z)]
    assert epath('/[:]/[:]', expr) == [(x, 1, t), 2, (3, y, 4), z]
    assert epath('/[:]/[:]/[:]', expr) == [x, 1, t, 3, y, 4]
    assert epath('/[:]/[:]/[:]/[:]', expr) == []

    assert epath('/*/[:]', expr) == [(x, 1, t), 2, (3, y, 4), z]

    assert epath('/*/[0]', expr) == [(x, 1, t), (3, y, 4)]
    assert epath('/*/[1]', expr) == [2, z]
    assert epath('/*/[2]', expr) == []

    assert epath('/*/int', expr) == [2]
    assert epath('/*/Symbol', expr) == [z]
    assert epath('/*/tuple', expr) == [(x, 1, t), (3, y, 4)]
    assert epath('/*/__iter__?', expr) == [(x, 1, t), (3, y, 4)]

    assert epath('/*/int|tuple', expr) == [(x, 1, t), 2, (3, y, 4)]
    assert epath('/*/Symbol|tuple', expr) == [(x, 1, t), (3, y, 4), z]
    assert epath('/*/int|Symbol|tuple', expr) == [(x, 1, t), 2, (3, y, 4), z]

    assert epath('/*/int|__iter__?', expr) == [(x, 1, t), 2, (3, y, 4)]
    assert epath('/*/Symbol|__iter__?', expr) == [(x, 1, t), (3, y, 4), z]
    assert epath(
        '/*/int|Symbol|__iter__?', expr) == [(x, 1, t), 2, (3, y, 4), z]

    assert epath('/*/[0]/int', expr) == [1, 3, 4]
    assert epath('/*/[0]/Symbol', expr) == [x, t, y]

    assert epath('/*/[0]/int[1:]', expr) == [1, 4]
    assert epath('/*/[0]/Symbol[1:]', expr) == [t, y]

    assert epath('/Symbol', x + y + z + 1) == [x, y, z]
    assert epath('/*/*/Symbol', t + sin(x + 1) + cos(x + y + E)) == [x, x, y]


def test_epath_apply():
    expr = [((x, 1, t), 2), ((3, y, 4), z)]

    def func(expr):
        return expr**2

    assert epath('/*', expr, list) == [[(x, 1, t), 2], [(3, y, 4), z]]

    assert epath('/*/[0]', expr, list) == [([x, 1, t], 2), ([3, y, 4], z)]
    assert epath('/*/[1]', expr, func) == [((x, 1, t), 4), ((3, y, 4), z**2)]
    assert epath('/*/[2]', expr, list) == expr

    assert epath('/*/[0]/int', expr, func) == [((x, 1, t), 2), ((9, y, 16), z)]
    assert epath('/*/[0]/Symbol', expr, func) == [((x**2, 1, t**2), 2),
                                                  ((3, y**2, 4), z)]
    assert epath(
        '/*/[0]/int[1:]', expr, func) == [((x, 1, t), 2), ((3, y, 16), z)]
    assert epath('/*/[0]/Symbol[1:]', expr, func) == [((x, 1, t**2),
                                                       2), ((3, y**2, 4), z)]

    assert epath('/Symbol', x + y + z + 1, func) == x**2 + y**2 + z**2 + 1
    assert epath('/*/*/Symbol', t + sin(x + 1) + cos(x + y + E), func) == \
        t + sin(x**2 + 1) + cos(x**2 + y**2 + E)
    assert epath('/*/*/Symbol', t*sin(x + 1)*cos(x + y + E), func) == \
        t*sin(x**2 + 1)*cos(x**2 + y**2 + E)
    assert epath('/*/*/Symbol', [((x, 1), 2), ((3, y), z)], func) == \
        [((x**2, 1), 2), ((3, y**2), z)]


def test_EPath():
    assert EPath('/*/[0]')._path == '/*/[0]'
    assert EPath(EPath('/*/[0]'))._path == '/*/[0]'
    assert isinstance(epath('/*/[0]'), EPath) is True

    assert repr(EPath('/*/[0]')) == "EPath('/*/[0]')"

    pytest.raises(ValueError, lambda: EPath(''))
    pytest.raises(ValueError, lambda: EPath('/'))
    pytest.raises(ValueError, lambda: EPath('/|x'))
    pytest.raises(ValueError, lambda: EPath('/['))
    pytest.raises(ValueError, lambda: EPath('/[0]%'))

    pytest.raises(NotImplementedError, lambda: EPath('Symbol'))



import pytest

from diofant import (E, Float, I, Rational, Symbol, ceiling, exp, factorial,
                     false, floor, log, nan, oo, pi, sin, symbols, true)
from diofant.abc import x, y


__all__ = ()


def test_floor():
    i = Symbol('i', imaginary=True)
    y = Symbol('y', extended_real=True)
    r = Symbol('r', real=True)
    k, n = symbols('k,n', integer=True)

    assert floor(y).is_extended_real
    assert floor(x).is_extended_real is None
    assert floor(r).is_finite
    assert floor(y).is_finite is None
    assert floor(r).is_integer
    assert floor(y).is_integer is None

    assert floor(nan) == nan

    assert floor(oo) == oo
    assert floor(-oo) == -oo

    assert floor(0) == 0

    assert floor(1) == 1
    assert floor(-1) == -1

    assert floor(E) == 2
    assert floor(-E) == -3

    assert floor(2*E) == 5
    assert floor(-2*E) == -6

    assert floor(pi) == 3
    assert floor(-pi) == -4

    assert floor(Rational(1, 2)) == 0
    assert floor(-Rational(1, 2)) == -1

    assert floor(Rational(7, 3)) == 2
    assert floor(-Rational(7, 3)) == -3

    assert floor(Float(17.0)) == 17
    assert floor(-Float(17.0)) == -17

    assert floor(Float(7.69)) == 7
    assert floor(-Float(7.69)) == -8

    assert floor(I) == I
    assert floor(-I) == -I
    e = floor(i)
    assert isinstance(e, floor) and e.args[0] == i

    assert floor(oo*I) == oo*I
    assert floor(-oo*I) == -oo*I

    assert floor(2*I) == 2*I
    assert floor(-2*I) == -2*I

    assert floor(I/2) == 0
    assert floor(-I/2) == -I

    assert floor(E + 17) == 19
    assert floor(pi + 2) == 5

    assert floor(E + pi) == floor(E + pi)
    assert floor(I + pi) == floor(I + pi)

    assert floor(floor(pi)) == 3
    assert floor(floor(y)) == floor(y)
    assert floor(floor(x)) == floor(floor(x))

    assert floor(x) == floor(x)
    assert floor(2*x) == floor(2*x)
    assert floor(k*x) == floor(k*x)

    assert floor(k) == k
    assert floor(2*k) == 2*k
    assert floor(k*n) == k*n

    assert floor(k/2) == floor(k/2)

    assert floor(x + y) == floor(x + y)

    assert floor(x + 3) == floor(x + 3)
    assert floor(x + k) == floor(x + k)

    assert floor(y + 3) == floor(y) + 3
    assert floor(y + k) == floor(y) + k

    assert floor(3 + I*y + pi) == 6 + floor(y)*I

    assert floor(k + n) == k + n

    assert floor(x*I) == floor(x*I)
    assert floor(k*I) == k*I

    assert floor(Rational(23, 10) - E*I) == 2 - 3*I

    assert floor(sin(1)) == 0
    assert floor(sin(-1)) == -1

    assert floor(exp(2)) == 7

    assert floor(log(8)/log(2)) != 2
    assert int(floor(log(8)/log(2)).evalf(chop=True)) == 3

    assert floor(factorial(50)/exp(1)) == \
        11188719610782480504630258070757734324011354208865721592720336800

    assert (floor(y) <= y) is true
    assert (floor(y) > y) is false
    assert (floor(x) <= x).is_Relational  # x could be non-real
    assert (floor(x) > x).is_Relational
    assert (floor(x) <= y).is_Relational  # arg is not same as rhs
    assert (floor(x) > y).is_Relational

    assert floor(x).as_leading_term(x) == floor(x)


def test_ceiling():
    i = Symbol('i', imaginary=True)
    y = Symbol('y', extended_real=True)
    k, n = symbols('k,n', integer=True)

    assert ceiling(nan) == nan

    assert ceiling(oo) == oo
    assert ceiling(-oo) == -oo

    assert ceiling(0) == 0

    assert ceiling(1) == 1
    assert ceiling(-1) == -1

    assert ceiling(E) == 3
    assert ceiling(-E) == -2

    assert ceiling(2*E) == 6
    assert ceiling(-2*E) == -5

    assert ceiling(pi) == 4
    assert ceiling(-pi) == -3

    assert ceiling(Rational(1, 2)) == 1
    assert ceiling(-Rational(1, 2)) == 0

    assert ceiling(Rational(7, 3)) == 3
    assert ceiling(-Rational(7, 3)) == -2

    assert ceiling(Float(17.0)) == 17
    assert ceiling(-Float(17.0)) == -17

    assert ceiling(Float(7.69)) == 8
    assert ceiling(-Float(7.69)) == -7

    assert ceiling(I) == I
    assert ceiling(-I) == -I
    e = ceiling(i)
    assert isinstance(e, ceiling) and e.args[0] == i

    assert ceiling(oo*I) == oo*I
    assert ceiling(-oo*I) == -oo*I

    assert ceiling(2*I) == 2*I
    assert ceiling(-2*I) == -2*I

    assert ceiling(I/2) == I
    assert ceiling(-I/2) == 0

    assert ceiling(E + 17) == 20
    assert ceiling(pi + 2) == 6

    assert ceiling(E + pi) == ceiling(E + pi)
    assert ceiling(I + pi) == ceiling(I + pi)

    assert ceiling(ceiling(pi)) == 4
    assert ceiling(ceiling(y)) == ceiling(y)
    assert ceiling(ceiling(x)) == ceiling(ceiling(x))

    assert ceiling(x) == ceiling(x)
    assert ceiling(2*x) == ceiling(2*x)
    assert ceiling(k*x) == ceiling(k*x)

    assert ceiling(k) == k
    assert ceiling(2*k) == 2*k
    assert ceiling(k*n) == k*n

    assert ceiling(k/2) == ceiling(k/2)

    assert ceiling(x + y) == ceiling(x + y)

    assert ceiling(x + 3) == ceiling(x + 3)
    assert ceiling(x + k) == ceiling(x + k)

    assert ceiling(y + 3) == ceiling(y) + 3
    assert ceiling(y + k) == ceiling(y) + k

    assert ceiling(3 + pi + y*I) == 7 + ceiling(y)*I

    assert ceiling(k + n) == k + n

    assert ceiling(x*I) == ceiling(x*I)
    assert ceiling(k*I) == k*I

    assert ceiling(Rational(23, 10) - E*I) == 3 - 2*I

    assert ceiling(sin(1)) == 1
    assert ceiling(sin(-1)) == 0

    assert ceiling(exp(2)) == 8

    assert ceiling(-log(8)/log(2)) != -2
    assert int(ceiling(-log(8)/log(2)).evalf(chop=True)) == -3

    assert ceiling(factorial(50)/exp(1)) == \
        11188719610782480504630258070757734324011354208865721592720336801

    assert (ceiling(y) >= y) is true
    assert (ceiling(y) < y) is false
    assert (ceiling(x) >= x).is_Relational  # x could be non-real
    assert (ceiling(x) < x).is_Relational
    assert (ceiling(x) >= y).is_Relational  # arg is not same as rhs
    assert (ceiling(x) < y).is_Relational


def test_series():
    assert floor(x).series(x, y, 100) == floor(y)
    assert ceiling(x).series(x, y, 100) == ceiling(y)
    assert floor(x).series(x, pi, 100) == 3
    assert ceiling(x).series(x, pi, 100) == 4
    assert floor(x).nseries(x, 100) == 0
    assert ceiling(x).nseries(x, 100) == 1
    assert floor(-x).nseries(x, 100) == -1
    assert ceiling(-x).nseries(x, 100) == 0


@pytest.mark.xfail
def test_sympyissue_4149():
    y = Symbol('y', real=True)
    assert floor(3 + pi*I + y*I) == 3 + floor(pi + y)*I
    assert floor(3*I + pi*I + y*I) == floor(3 + pi + y)*I
    assert floor(3 + E + pi*I + y*I) == 5 + floor(pi + y)*I


def test_sympyissue_11207():
    assert floor(floor(x)) == floor(x)
    assert floor(ceiling(x)) == ceiling(x)
    assert ceiling(floor(x)) == floor(x)
    assert ceiling(ceiling(x)) == ceiling(x)



"""Implicit plotting module for Diofant

The module implements a data series called ImplicitSeries which is used by
``Plot`` class to plot implicit plots for different backends.

See Also
========

diofant.plotting.plot

"""

from ..core import Dummy, Eq, Symbol, Tuple, sympify
from ..core.relational import Equality, GreaterThan, LessThan, Relational
from ..external import import_module
from ..logic.boolalg import BooleanFunction
from ..polys.polyutils import _sort_gens
from ..utilities import flatten, lambdify
from ..utilities.decorator import doctest_depends_on
from .plot import BaseSeries, Plot


class ImplicitSeries(BaseSeries):
    """Representation for Implicit plot."""

    is_implicit = True

    def __init__(self, expr, var_start_end_x, var_start_end_y,
                 has_equality, use_interval_math, depth, nb_of_points,
                 line_color):
        super().__init__()
        self.expr = sympify(expr)
        self.var_x = sympify(var_start_end_x[0])
        self.start_x = float(var_start_end_x[1])
        self.end_x = float(var_start_end_x[2])
        self.var_y = sympify(var_start_end_y[0])
        self.start_y = float(var_start_end_y[1])
        self.end_y = float(var_start_end_y[2])
        self.get_points = self.get_raster

        # If the expression has equality, i.e. Eq, Greaterthan, LessThan.
        self.has_equality = has_equality

        self.nb_of_points = nb_of_points
        self.use_interval_math = use_interval_math
        self.depth = 4 + depth
        self.line_color = line_color

    def get_raster(self):
        return self._get_meshes_grid()

    def _get_meshes_grid(self):
        """Generates the mesh for generating a contour.

        In the case of equality, ``contour`` function of matplotlib can
        be used. In other cases, matplotlib's ``contourf`` is used.

        """
        equal = False
        if isinstance(self.expr, Equality):
            expr = self.expr.lhs - self.expr.rhs
            equal = True
        elif self.expr.has(Equality):  # pragma: no cover
            raise NotImplementedError('The expression is not supported for '
                                      'plotting in uniform meshed plot.')
        else:
            expr = self.expr
        np = import_module('numpy')
        xarray = np.linspace(self.start_x, self.end_x, self.nb_of_points)
        yarray = np.linspace(self.start_y, self.end_y, self.nb_of_points)
        x_grid, y_grid = np.meshgrid(xarray, yarray)

        func = lambdify((self.var_x, self.var_y), expr, 'numpy')
        z_grid = func(x_grid, y_grid)
        z_grid[np.ma.where(z_grid < 0)] = -1
        z_grid[np.ma.where(z_grid > 0)] = 1
        if equal:
            return xarray, yarray, z_grid, 'contour'
        else:
            return xarray, yarray, z_grid, 'contourf'


@doctest_depends_on(modules=('matplotlib',))
def plot_implicit(expr, x_var=None, y_var=None, **kwargs):
    """A plot function to plot implicit equations / inequalities.

    Parameters
    ==========

    ``expr`` : Expr
        The equation / inequality that is to be plotted.
    ``x_var`` : symbol or tuple, optional
        symbol to plot on x-axis or tuple giving symbol and range
        as ``(symbol, xmin, xmax)``
    ``y_var`` : symbol or tuple, optional
        symbol to plot on y-axis or tuple giving symbol and range
        as ``(symbol, ymin, ymax)``

    If neither ``x_var`` nor ``y_var`` are given then the free symbols in the
    expression will be assigned in the order they are sorted.

    The following keyword arguments can also be used:

    ``adaptive`` : Boolean, optional
        The default value is set to True. It has to be
        set to False if you want to use a mesh grid.

    ``depth`` : integer
        The depth of recursion for adaptive mesh grid.
        Default value is 0. Takes value in the range (0, 4).

    ``points`` : integer
        The number of points if adaptive mesh grid is not
        used. Default value is 200.

    ``title`` : str
        The title for the plot.

    ``xlabel`` : str
        The label for the x-axis

    ``ylabel`` : string
        The label for the y-axis

    Aesthetics options:

    ``line_color`` : float or str
        Specifies the color for the plot.  See ``Plot`` to see how to
        set color for the plots.

    plot_implicit, by default, uses interval arithmetic to plot functions. If
    the expression cannot be plotted using interval arithmetic, it defaults to
    a generating a contour using a mesh grid of fixed number of points. By
    setting adaptive to False, you can force plot_implicit to use the mesh
    grid. The mesh grid method can be effective when adaptive plotting using
    interval arithmetic, fails to plot with small line width.

    Examples
    ========

    Plot expressions:

    Without any ranges for the symbols in the expression

    >>> p1 = plot_implicit(Eq(x**2 + y**2, 5))

    With the range for the symbols

    >>> p2 = plot_implicit(Eq(x**2 + y**2, 3),
    ...                    (x, -3, 3), (y, -3, 3))

    With depth of recursion as argument.

    >>> p3 = plot_implicit(Eq(x**2 + y**2, 5),
    ...                    (x, -4, 4), (y, -4, 4), depth=2)

    Using mesh grid and not using adaptive meshing.

    >>> p4 = plot_implicit(Eq(x**2 + y**2, 5),
    ...                    (x, -5, 5), (y, -2, 2), adaptive=False)

    Using mesh grid with number of points as input.

    >>> p5 = plot_implicit(Eq(x**2 + y**2, 5),
    ...                    (x, -5, 5), (y, -2, 2),
    ...                    adaptive=False, points=400)

    Plotting regions.

    >>> p6 = plot_implicit(y > x**2)

    Plotting Using boolean conjunctions.

    >>> p7 = plot_implicit(And(y > x, y > -x))

    When plotting an expression with a single variable (y - 1, for example),
    specify the x or the y variable explicitly:

    >>> p8 = plot_implicit(y - 1, y_var=y)
    >>> p9 = plot_implicit(x - 1, x_var=x)

    """
    # Represents whether the expression contains an Equality,
    # GreaterThan or LessThan
    has_equality = False

    def arg_expand(bool_expr):
        """Recursively expands the arguments of an Boolean Function."""
        for arg in bool_expr.args:
            if isinstance(arg, BooleanFunction):
                arg_expand(arg)
            elif isinstance(arg, Relational):
                arg_list.append(arg)

    arg_list = []
    if isinstance(expr, BooleanFunction):
        arg_expand(expr)

    # Check whether there is an equality in the expression provided.
        if any(isinstance(e, (Equality, GreaterThan, LessThan))
               for e in arg_list):
            has_equality = True

    elif not isinstance(expr, Relational):
        expr = Eq(expr, 0)
        has_equality = True
    elif isinstance(expr, (Equality, GreaterThan, LessThan)):
        has_equality = True

    xyvar = [i for i in (x_var, y_var) if i is not None]
    free_symbols = expr.free_symbols
    range_symbols = Tuple(*flatten(xyvar)).free_symbols
    undeclared = free_symbols - range_symbols
    if len(free_symbols & range_symbols) > 2:  # pragma: no cover
        raise NotImplementedError('Implicit plotting is not implemented for '
                                  'more than 2 variables')

    # Create default ranges if the range is not provided.
    default_range = Tuple(-5, 5)

    def _range_tuple(s):
        if isinstance(s, (Dummy, Symbol)):
            return Tuple(s) + default_range
        if len(s) == 3:
            return Tuple(*s)
        raise ValueError(f'symbol or `(symbol, min, max)` expected but got {s!s}')

    if len(xyvar) == 0:
        xyvar = list(_sort_gens(free_symbols))
    var_start_end_x = _range_tuple(xyvar[0])
    x = var_start_end_x[0]
    if len(xyvar) != 2:
        if x in undeclared or not undeclared:
            xyvar.append(Dummy(f'f({x.name})'))
        else:
            xyvar.append(undeclared.pop())
    var_start_end_y = _range_tuple(xyvar[1])

    use_interval = kwargs.pop('adaptive', False)
    nb_of_points = kwargs.pop('points', 300)
    depth = kwargs.pop('depth', 0)
    line_color = kwargs.pop('line_color', 'blue')
    # Check whether the depth is greater than 4 or less than 0.
    if depth > 4:
        depth = 4
    elif depth < 0:
        depth = 0

    series_argument = ImplicitSeries(expr, var_start_end_x, var_start_end_y,
                                     has_equality, use_interval, depth,
                                     nb_of_points, line_color)
    show = kwargs.pop('show', True)

    # set the x and y limits
    kwargs['xlim'] = tuple(float(x) for x in var_start_end_x[1:])
    kwargs['ylim'] = tuple(float(y) for y in var_start_end_y[1:])
    # set the x and y labels
    kwargs.setdefault('xlabel', var_start_end_x[0].name)
    kwargs.setdefault('ylabel', var_start_end_y[0].name)
    p = Plot(series_argument, **kwargs)
    if show:
        p.show()
    return p



import pytest

from diofant import (Eq, Function, I, Lambda, binomial, cos, factorial, gamma,
                     pi, rf, rsolve, sin, sqrt, symbols)
from diofant.abc import a, b
from diofant.solvers.recurr import rsolve_hyper, rsolve_poly, rsolve_ratio


__all__ = ()

f, g = symbols('f,g', cls=Function)
n, k = symbols('n,k', integer=True)
C0, C1, C2 = symbols('C:3')


def test_poly():
    assert rsolve_poly([-1, -1, 1], 0, n) == (0, [])
    assert rsolve_poly([-1, -1, 1], 1, n) == (-1, [])
    assert rsolve_poly([-n**2, n, -1, 1], 1, n) is None

    assert rsolve(-f(n) + (n + 1)*f(n + 1) -
                  n) == [{f: Lambda(n, C0/factorial(n) + 1)}]
    assert rsolve(-(n + 1)*f(n) + n*f(n + 1) -
                  1) == [{f: Lambda(n, C0*n - 1)}]
    assert rsolve(-(4*n + 2)*f(n) + f(n + 1) - 4*n -
                  1) == [{f: Lambda(n, 4**n*C0*gamma(n + 1/2)/sqrt(pi) - 1)}]

    assert (rsolve(-f(n) + f(n + 1) - n**5 - n**3) ==
            [{f: Lambda(n, C0 + n**2*(n**4 - 3*n**3 + 4*n**2 - 3*n + 1)/6)}])

    assert rsolve_poly([1, 1], sqrt(n), n) is None

    assert rsolve_poly([-2, -1, 1],
                       -2*n**4 - (n + 1)**4 + (n + 2)**4, n) == (n**4, [])
    assert rsolve(n*f(n) - f(n + 1) - n**3 +
                  (n + 1)**2) == [{f: Lambda(n, (C0*factorial(n) + n**3)/n)}]


def test_ratio():
    assert rsolve_ratio([-2*n**3 + n**2 + 2*n - 1, 2*n**3 + n**2 - 6*n,
                         -2*n**3 - 11*n**2 - 18*n - 9, 2*n**3 + 13*n**2 +
                         22*n + 8], 0, n) == (C2*(2*n - 3)/(n**2 - 1)/2, [C2])
    assert rsolve_ratio([1, 1], sqrt(n), n) is None
    assert rsolve_ratio([-n**3, n + 1], n, n) is None


def test_hyper():
    assert rsolve((n**2 - 2)*f(n) - (2*n + 1)*f(n + 1) +
                  f(n + 2)) == [{f: Lambda(n, C0*rf(-sqrt(2), n) +
                                           C1*rf(+sqrt(2), n))}]

    assert rsolve((n**2 - k)*f(n) - (2*n + 1)*f(n + 1) +
                  f(n + 2)) == [{f: Lambda(n, C1*rf(sqrt(k), n) +
                                           C0*rf(-sqrt(k), n))}]

    assert rsolve(2*n*(n + 1)*f(n) - (n**2 + 3*n - 2)*f(n + 1) +
                  (n - 1)*f(n + 2)) == [{f: Lambda(n, C1*factorial(n) +
                                                   C0*2**n)}]

    assert rsolve_hyper([n + 2, -(2*n + 3)*(17*n**2 + 51*n + 39), n + 1],
                        0, n) == (0, [])

    assert rsolve_hyper([-n - 1, -1, 1], 0, n) == (0, [])

    assert rsolve(-n - f(n) + f(n + 1)) == [{f: Lambda(n, C0 + n*(n - 1)/2)}]
    assert rsolve(-1 - n - f(n) +
                  f(n + 1)) == [{f: Lambda(n, C0 + n*(n + 1)/2)}]
    assert rsolve(-3*(n + n**2) - f(n) +
                  f(n + 1)) == [{f: Lambda(n, C0 + n**3 - n)}]

    assert rsolve(-n - factorial(n) - f(n) + f(n + 1)) is None

    assert rsolve(-a*f(n) + f(n + 1)) == [{f: Lambda(n, C0*a**n)}]
    assert rsolve(-a*f(n) +
                  f(n + 2)) == [{f: Lambda(n, a**(n/2)*((-1)**n*C1 + C0))}]

    assert (rsolve(f(n) + f(n + 1) + f(n + 2)) ==
            [{f: Lambda(n, 2**-n*(C0*(-1 - sqrt(3)*I)**n +
                                  C1*(-1 + sqrt(3)*I)**n))}])

    assert rsolve_hyper([1, -2*n/a - 2/a, 1], 0, n) == (0, [])

    assert rsolve_hyper([1, 1], sqrt(n), n) is None
    assert rsolve_hyper([1, 1], n + sqrt(n), n) is None


@pytest.mark.slow
def test_bulk():
    funcs = [n, n + 1, n**2, n**3, n**4, n + n**2,
             27*n + 52*n**2 - 3*n**3 + 12*n**4 - 52*n**5]
    coeffs = [[-2, 1], [-2, -1, 1], [-1, 1, 1, -1, 1],
              [-n, 1], [n**2 - n + 12, 1]]
    for p in funcs:
        for c in coeffs:
            q = sum(c[i]*p.subs({n: n + i}) for i in range(len(c)))
            if p.is_polynomial(n):
                assert rsolve_poly(c, q, n)[0] == p
            if p.is_hypergeometric(n) and len(c) <= 3:
                assert rsolve_hyper(c, q, n)[0].subs({C0: 0, C1: 0,
                                                      C2: 0}).expand() == p


def test_rsolve():
    eq = f(n + 2) - f(n + 1) - f(n)
    res = [{f: Lambda(n, 2**(-n)*(C0*(1 + sqrt(5))**n +
                                  C1*(-sqrt(5) + 1)**n))}]

    assert rsolve(eq) == res

    res = [{k: v.subs({C0: sqrt(5), C1: -sqrt(5)}).simplify()
            for k, v in r.items()} for r in res]

    assert rsolve(eq, init={f(0): 0, f(1): 5}) == res
    assert rsolve(f(n) - f(n - 1) - f(n - 2), init={f(0): 0, f(1): 5}) == res
    assert rsolve(Eq(f(n), f(n - 1) + f(n - 2)), init={f(0): 0, f(1): 5}) == res

    eq = (n - 1)*f(n + 2) - (n**2 + 3*n - 2)*f(n + 1) + 2*n*(n + 1)*f(n)
    res = [{f: Lambda(n, C1*factorial(n) + C0*2**n)}]

    assert rsolve(eq) == res

    res = [{f: Lambda(n, -3*factorial(n) + 3*2**n)}]

    assert rsolve(eq, init={f(0): 0, f(1): 3}) == res

    eq = f(n) - f(n - 1) - 2

    assert rsolve(eq, f(n)) == [{f: Lambda(n, C0 + 2*n)}]
    assert rsolve(eq) == [{f: Lambda(n, C0 + 2*n)}]
    assert rsolve(eq, init={f(0): 0}) == [{f: Lambda(n, 2*n)}]
    assert rsolve(eq, init={f(0): 1}) == [{f: Lambda(n, 2*n + 1)}]
    assert rsolve(eq, init={f(0): 0, f(1): 1}) is None

    eq = 3*f(n - 1) - f(n) - 1

    assert rsolve(eq) == [{f: Lambda(n, 3**n*C0 + 1/2)}]
    assert rsolve(eq, init={f(0): 0}) == [{f: Lambda(n, -3**n/2 + 1/2)}]
    assert rsolve(eq, init={f(0): 1}) == [{f: Lambda(n, 3**n/2 + 1/2)}]
    assert rsolve(eq, init={f(0): 2}) == [{f: Lambda(n, 3*3**n/2 + 1/2)}]

    assert rsolve(f(n) - 1/n*f(n - 1),
                  f(n)) == [{f: Lambda(n, C0/factorial(n))}]
    assert rsolve(f(n) - 1/n*f(n - 1) - 1, f(n)) is None

    eq = 2*f(n - 1) + (1 - n)*f(n)/n

    assert rsolve(eq) == [{f: Lambda(n, 2**n*C0*n)}]
    assert rsolve([eq]) == [{f: Lambda(n, 2**n*C0*n)}]
    assert rsolve(eq, init={f(1): 1}) == [{f: Lambda(n, 2**(n - 1)*n)}]
    assert rsolve(eq, init={f(1): 2},
                  simplify=False) == [{f: Lambda(n, 2**(n - 1)*n*2)}]
    assert rsolve(eq, init={f(1): 2}) == [{f: Lambda(n, 2**n*n)}]
    assert rsolve(eq, init={f(1): 3}) == [{f: Lambda(n, 3*2**n*n/2)}]

    eq = (n - 1)*(n - 2)*f(n + 2) - (n + 1)*(n + 2)*f(n)

    assert rsolve(eq) == [{f: Lambda(n,
                                     n*(n - 2)*(n - 1)*((-1)**n*C1 + C0))}]
    assert rsolve(eq, init={f(3): 6,
                            f(4): 24}) == [{f: Lambda((n), n*(n - 1)*(n - 2))}]
    assert (rsolve(eq, init={f(3): 6, f(4): -24}) ==
            [{f: Lambda(n, (-1)**(n + 1)*n*(n - 2)*(n - 1))}])

    assert rsolve(Eq(f(n + 1), a*f(n)),
                  init={f(1): a}) == [{f: Lambda(n, a**n)}]

    assert (rsolve(f(n) - a*f(n - 2),
                   init={f(1): sqrt(a)*(a + b), f(2): a*(a - b)}) ==
            [{f: Lambda(n, a**(n/2)*(-(-1)**n*b + a))}])

    eq = (-16*n**2 + 32*n - 12)*f(n - 1) + (4*n**2 - 12*n + 9)*f(n)

    assert (rsolve(eq, init={f(1): binomial(2*n + 1, 3)}) ==
            [{f: Lambda(n, 4**n*n*(2*n - 1)*gamma(n + 3/2)/(3*gamma(n - 1/2)))}])

    assert (rsolve(f(n) + a*(f(n + 1) + f(n - 1))/2) ==
            [{f: Lambda(n, a**-n*(C0*(-a*sqrt(-1 + a**-2) - 1)**n +
                                  C1*(a*sqrt(-1 + a**-2) - 1)**n))}])


def test_rsolve_raises():
    pytest.raises(ValueError, lambda: rsolve(f(n) - f(k + 1), f(n)))
    pytest.raises(ValueError, lambda: rsolve(f(n) - sqrt(n)*f(n + 1)))
    pytest.raises(ValueError,
                  lambda: rsolve(f(n) - f(n + 1), f(n), init={g(0): 0}))
    pytest.raises(NotImplementedError, lambda: rsolve(f(n) - sqrt(n)))


def test_sympyissue_6844():
    eq = f(n + 2) - f(n + 1) + f(n)/4

    assert rsolve(eq) == [{f: Lambda(n, 2**(-n)*(C0 + C1*n))}]
    assert rsolve(eq, init={f(0): 0,
                            f(1): 1}) == [{f: Lambda(n, 2**(1 - n)*n)}]


def test_diofantissue_294():
    eq = f(n) - f(n - 1) - 2*f(n - 2) - 2*n

    assert rsolve(eq) == [{f: Lambda(n, (-1)**n*C0 + 2**n*C1 - n - 5/2)}]
    # issue sympy/sympy#11261
    assert rsolve(eq, init={f(0): -1,
                            f(1): 1}) == [{f: Lambda(n, -(-1)**n/2 +
                                                     2**(n + 1) - n - 5/2)}]
    # issue sympy/sympy#7055
    assert rsolve(-2*f(n) + f(n + 1) +
                  n - 1) == [{f: Lambda(n, 2**n*C0 + n)}]


def test_sympyissue_8697():
    assert rsolve(f(n + 3) - f(n + 2) - f(n + 1) +
                  f(n)) == [{f: Lambda(n, (-1)**n*C1 + C0 + C2*n)}]
    assert (rsolve(f(n + 3) + 3*f(n + 2) + 3*f(n + 1) + f(n)) ==
            [{f: Lambda(n, (-1)**n*(C0 + C1*n + C2*n**2))}])

    assert rsolve(f(n) - 2*f(n - 3) + 5*f(n - 2) - 4*f(n - 1),
                  init={f(0): 1, f(1): 3,
                        f(2): 8}) == [{f: Lambda(n, 3*2**n - n - 2)}]

    # From issue thread (but not related to the problem, fixed before):
    assert rsolve(f(n) - 2*f(n - 1) - n,
                  init={f(0): 1}) == [{f: Lambda(n, 3*2**n - n - 2)}]
    assert (rsolve(f(n + 2) - 5*f(n + 1) + 6*f(n) - n) ==
            [{f: Lambda(n, 2**n*C0 + 3**n*C1 + n/2 + 3/4)}])


def test_diofantissue_451():
    assert rsolve(f(n) - 2*f(n - 1) - 3**n,
                  init={f(0): 1}) == [{f: Lambda(n, -2**(n + 1) +
                                                 3**(n + 1))}]


def test_diofantissue_456():
    assert rsolve(f(n) - 2*f(n - 1) - 3**n*n,
                  init={f(0): 1}) == [{f: Lambda(n, 7*2**n +
                                                 3**(n + 1)*(n - 2))}]


def test_diofantissue_13629():
    assert rsolve(f(n + 1) - (f(n) + (n + 1)**2),
                  init={f(0): 0}) == [{f: Lambda(n, n*(2*n**2 + 3*n + 1)/6)}]


def test_sympyissue_15553():
    assert rsolve(Eq(f(n + 1), 2*f(n) +
                     n**2 + 1)) == [{f: Lambda(n, 2**n*C0 - n**2 - 2*n - 4)}]
    assert rsolve(Eq(f(n + 1), 2*f(n) + n**2 + 1),
                  init={f(1): 0}) == [{f: Lambda(n, 7*2**n/2 - n**2 -
                                                 2*n - 4)}]


def test_diofantissue_922():
    assert rsolve(-2*n/3 + f(n) - f(n - 1) + 2*(n - 1)**3/3 + 2*(n - 1)**2/3,
                  init={f(0): 0}) == [{f: Lambda(n, n*(-3*n**3 + 2*n**2 +
                                                       9*n + 4)/18)}]


def test_diofantissue_923():
    assert rsolve(4*f(n) + 4*f(n + 1) +
                  f(n + 2)) == [{f: Lambda(n, (-2)**n*(C0 + C1*n))}]


def test_sympyissue_17982():
    assert (rsolve(f(n + 3) + 10*f(n + 2) + 32*f(n + 1) + 32*f(n)) ==
            [{f: Lambda(n, (-2)**n*C0 + (-4)**n*C1 + (-4)**n*C2*n)}])


def test_sympyissue_18751():
    r = symbols('r', real=True, positive=True)
    theta = symbols('theta', real=True)

    eq = f(n) - 2*r*cos(theta)*f(n - 1) + r**2*f(n - 2)
    res = [{f: Lambda(n, r**n*(C0*(cos(theta) - I*abs(sin(theta)))**n +
                      C1*(cos(theta) + I*abs(sin(theta)))**n))}]

    assert rsolve(eq) == res



"""Solvers of systems of polynomial equations."""

import collections

from ..domains import EX
from ..matrices import Matrix
from ..polys import groebner, poly, sring
from ..polys.polyerrors import ComputationFailed, PolificationFailed
from ..polys.polytools import parallel_poly_from_expr
from ..polys.solvers import solve_lin_sys
from ..simplify import simplify
from ..utilities import default_sort_key, numbered_symbols
from .utils import checksol


__all__ = ('solve_linear_system', 'solve_poly_system',
           'solve_surd_system')


def solve_linear_system(system, *symbols, **flags):
    r"""Solve system of linear equations.

    Both under- and overdetermined systems are supported. The possible
    number of solutions is zero, one or infinite.

    Parameters
    ==========

    system : Matrix
        Nx(M+1) matrix, which means it has to be in augmented
        form.  This matrix will not be modified.
    \*symbols : list
        List of M Symbol's

    Returns
    =======

    solution: dict or None
        Respectively, this procedure will return None or
        a dictionary with solutions.  In the case of underdetermined
        systems, all arbitrary parameters are skipped.  This may
        cause a situation in which an empty dictionary is returned.
        In that case, all symbols can be assigned arbitrary values.

    Examples
    ========

    Solve the following system::

           x + 4 y ==  2
        -2 x +   y == 14

    >>> system = Matrix(((1, 4, 2), (-2, 1, 14)))
    >>> solve_linear_system(system, x, y)
    {x: -6, y: 2}

    A degenerate system returns an empty dictionary.

    >>> system = Matrix(((0, 0, 0), (0, 0, 0)))
    >>> solve_linear_system(system, x, y)
    {}

    See Also
    ========

    diofant.matrices.matrices.MatrixBase.rref

    """
    eqs = system*Matrix(symbols + (-1,))
    domain, eqs = sring(eqs.transpose().tolist()[0], *symbols, field=True)

    res = solve_lin_sys(eqs, domain)
    if res is None:
        return

    for k in list(res):
        s = domain.symbols[domain.index(k)]
        res[s] = res[k].as_expr()
        del res[k]
        if flags.get('simplify', True):
            res[s] = simplify(res[s])

    return res


def solve_poly_system(eqs, *gens, **args):
    """
    Solve a system of polynomial equations.

    Polynomial system may have finite number of solutions or
    infinitely many (positive-dimensional systems).

    References
    ==========

    * :cite:`Cox2015ideals`, p. 98

    Examples
    ========

    >>> solve_poly_system([x*y - 2*y, 2*y**2 - x**2], x, y)
    [{x: 0, y: 0}, {x: 2, y: -sqrt(2)}, {x: 2, y: sqrt(2)}]

    >>> solve_poly_system([x*y], x, y)
    [{x: 0}, {y: 0}]

    """
    try:
        args['extension'] = False
        polys, opt = parallel_poly_from_expr(eqs, *gens, **args)
        polys = [p.to_exact() for p in polys]
    except PolificationFailed as exc:
        raise ComputationFailed('solve_poly_system', len(eqs), exc)

    def _solve_reduced_system(system, gens):
        """Recursively solves reduced polynomial systems."""
        basis = groebner(system, gens, polys=True, extension=False)
        dim = basis.dimension
        solutions = []

        if dim is None:
            return []

        elif dim > 0:
            max_iset = max(basis.independent_sets, key=len)
            new_gens = [g for g in gens if g not in max_iset]

            # After removing variables from the maximal set of independent
            # variables for the given ideal - the new ideal is of dimension
            # zero with the independent variables as parameters in the
            # coefficient domain.
            solutions.extend(_solve_reduced_system(system, new_gens))

            # Now we should examine cases when leading coefficient of
            # some polynomial in the system is zero.
            for p in basis.polys:
                lc = poly(p, *new_gens).LC(order=basis.order)
                for special in _solve_reduced_system(system + [lc], gens):
                    # This heuristics wipe out some redundant special
                    # solutions, which already there in solutions after
                    # solving the system with new set of generators.
                    if all(any((_.subs(s) - _).subs(special).simplify()
                               for _ in gens) for s in solutions):
                        solutions.insert(0, special)

        else:
            # By the elimination property, the last polynomial should
            # be univariate in the last variable.
            f = basis[-1]
            gen = gens[-1]

            zeros = {k.doit() for k in f.exclude().all_roots()}

            if len(basis) == 1:
                return [{gen: zero} for zero in zeros]

            new_basis = [b.set_domain(EX) for b in basis[:-1]]

            # Now substitute zeros for the last variable and
            # solve recursively new obtained zero-dimensional systems.
            for zero in zeros:
                new_system = []
                new_gens = gens[:-1]

                for b in new_basis:
                    eq = b.eval(gen, zero)

                    if not eq.is_zero:
                        new_system.append(eq)

                for solution in _solve_reduced_system(new_system, new_gens):
                    solution[gen] = zero
                    solutions.append(solution)

        return solutions

    result = _solve_reduced_system(polys, opt.gens)

    if not opt.domain.is_Exact:
        result = [{k: r[k].evalf(opt.domain.dps) for k in r} for r in result]

    return sorted(result, key=default_sort_key)


def solve_surd_system(eqs, *gens, **args):
    """
    Solve a system of algebraic equations.

    Examples
    ========

    >>> solve_surd_system([x + sqrt(x + 1) - 2])
    [{x: -sqrt(13)/2 + 5/2}]

    """
    eqs = list(eqs)

    if not gens:
        gens = set().union(*[_.free_symbols for _ in eqs])
        gens = sorted(gens, key=default_sort_key)
    else:
        gens = list(gens)

    aux = numbered_symbols('a')
    neqs = len(eqs)
    orig_eqs = eqs[:]
    ngens = len(gens)
    bases = collections.defaultdict(dict)

    def q_surd(e):
        return e.is_Pow and e.exp.is_Rational and not e.exp.is_Integer

    def tr_surd(e):
        n, d = e.exp.as_numer_denom()
        for v2, d2 in sorted(bases.get(e.base, {}).items(),
                             key=lambda _: -_[1]):
            if not d2 % d:
                return v2**(d2 // d)
        v = next(aux)
        bases[e.base][v] = d
        gens.append(v)
        eqs.append(v**d - e.base)
        return v**n

    for i in range(neqs):
        eqs[i] = eqs[i].replace(q_surd, tr_surd)

    denoms = []
    for i, e in enumerate(eqs):
        eqs[i], d = e.as_numer_denom()
        if not d.is_constant(*gens):
            denoms.insert(0, d)

    weaksols = solve_poly_system(eqs, *gens, **args)

    for i in range(len(weaksols) - 1, -1, -1):
        if any(checksol(_, weaksols[i], warn=True) for _ in denoms):
            del weaksols[i]
        elif any(checksol(_, weaksols[i], warn=True) is False for _ in orig_eqs):
            del weaksols[i]
        else:
            for g in gens[ngens:]:
                del weaksols[i][g]

    return weaksols



# This testfile tests Diofant <-> SciPy compatibility

# Don't test any Diofant features here. Just pure interaction with SciPy.
# Always write regular Diofant tests for anything, that can be tested in pure
# Python (without scipy). Here we test everything, that a user may need when
# using Diofant with SciPy

import pytest

from diofant import jn_zeros


__all__ = ()

scipy = pytest.importorskip('scipy')


def eq(a, b, tol=1e-6):
    for x, y in zip(a, b):
        if not (abs(x - y) < tol):
            return False
    return True


def test_jn_zeros():
    assert eq(jn_zeros(0, 4, method='scipy'),
              [3.141592, 6.283185, 9.424777, 12.566370])
    assert eq(jn_zeros(1, 4, method='scipy'),
              [4.493409, 7.725251, 10.904121, 14.066193])
    assert eq(jn_zeros(2, 4, method='scipy'),
              [5.763459, 9.095011, 12.322940, 15.514603])
    assert eq(jn_zeros(3, 4, method='scipy'),
              [6.987932, 10.417118, 13.698023, 16.923621])
    assert eq(jn_zeros(4, 4, method='scipy'),
              [8.182561, 11.704907, 15.039664, 18.301255])



"""
A Printer for generating readable representation of most diofant classes.
"""

import mpmath.libmp as mlib
from mpmath.libmp import prec_to_dps

from ..core import Integer, Mul, Pow, Rational, S, oo
from ..core.mul import _keep_coeff
from ..utilities import default_sort_key
from .defaults import DefaultPrinting
from .precedence import PRECEDENCE, precedence
from .printer import Printer


class StrPrinter(Printer):
    """Str printer."""

    printmethod = '_diofantstr'
    _default_settings = {
        'order': None,
        'full_prec': 'auto',
    }

    _relationals = {}

    def parenthesize(self, item, level):
        if precedence(item) <= level:
            return '(%s)' % self._print(item)
        else:
            return self._print(item)

    def stringify(self, args, sep, level=0):
        return sep.join([self.parenthesize(item, level) for item in args])

    def emptyPrinter(self, expr):
        if isinstance(expr, str):
            return expr
        elif hasattr(expr, '__str__') and not issubclass(expr.__class__,
                                                         DefaultPrinting):
            return str(expr)
        else:
            return repr(expr)

    def _print_Add(self, expr, order=None):
        if self.order == 'none':
            terms = list(expr.args)
        else:
            terms = expr.as_ordered_terms(order=order or self.order)

        PREC = precedence(expr)
        l = []
        for term in terms:
            t = self._print(term)
            if t.startswith('-'):
                sign = '-'
                t = t[1:]
            else:
                sign = '+'
            if precedence(term) < PREC:
                l.extend([sign, f'({t})'])
            else:
                l.extend([sign, t])
        sign = l.pop(0)
        if sign == '+':
            sign = ''
        return sign + ' '.join(l)

    def _print_BooleanTrue(self, expr):
        return 'true'

    def _print_BooleanFalse(self, expr):
        return 'false'

    def _print_Not(self, expr):
        return '~%s' % self.parenthesize(expr.args[0], PRECEDENCE['Not'])

    def _print_And(self, expr):
        return self.stringify(expr.args, ' & ', PRECEDENCE['BitwiseAnd'])

    def _print_Or(self, expr):
        return self.stringify(expr.args, ' | ', PRECEDENCE['BitwiseOr'])

    def _print_Basic(self, expr):
        l = [self._print(o) for o in expr.args]
        return expr.__class__.__name__ + '(%s)' % ', '.join(l)

    def _print_BlockMatrix(self, B):
        return self._print(B.blocks)

    def _print_Catalan(self, expr):
        return 'Catalan'

    def _print_ComplexInfinity(self, expr):
        return 'zoo'

    def _print_Derivative(self, expr):
        return 'Derivative(%s)' % ', '.join(map(self._print, expr.args))

    def _print_dict(self, d):
        keys = sorted(d, key=default_sort_key)
        items = []

        for key in keys:
            item = f'{self._print(key)}: {self._print(d[key])}'
            items.append(item)

        return '{%s}' % ', '.join(items)

    def _print_Dict(self, expr):
        return self._print_dict(expr)

    def _print_RandomDomain(self, d):
        return 'Domain: ' + self._print(d.as_boolean())

    def _print_Dummy(self, expr):
        return '_' + expr.name

    def _print_EulerGamma(self, expr):
        return 'EulerGamma'

    def _print_Exp1(self, expr):
        return 'E'

    def _print_ExprCondPair(self, expr):
        return f'({expr.expr}, {expr.cond})'

    def _print_FiniteSet(self, s):
        s = sorted(s, key=default_sort_key)
        if len(s) > 10:
            printset = s[:3] + ['...'] + s[-3:]
        else:
            printset = s
        return '{' + ', '.join(self._print(el) for el in printset) + '}'

    def _print_Function(self, expr):
        return expr.func.__name__ + '(%s)' % self.stringify(expr.args, ', ')

    def _print_GeometryEntity(self, expr):
        # GeometryEntity is special -- it's base is tuple
        return str(expr)

    def _print_GoldenRatio(self, expr):
        return 'GoldenRatio'

    def _print_ImaginaryUnit(self, expr):
        return 'I'

    def _print_Infinity(self, expr):
        return 'oo'

    def _print_Integral(self, expr):
        def _xab_tostr(xab):
            if len(xab) == 1:
                return self._print(xab[0])
            else:
                return self._print((xab[0],) + tuple(xab[1:]))
        L = ', '.join([_xab_tostr(l) for l in expr.limits])
        return f'Integral({self._print(expr.function)}, {L})'

    def _print_Interval(self, i):
        if i.left_open:
            left = '('
        else:
            left = '['

        if i.right_open:
            right = ')'
        else:
            right = ']'

        return f'{left}{self._print(i.start)}, {self._print(i.end)}{right}'

    def _print_Inverse(self, I):
        return '%s^-1' % self.parenthesize(I.arg, PRECEDENCE['Pow'])

    def _print_Lambda(self, obj):
        args, expr = obj.args
        if len(args) == 1:
            return f'Lambda({args.args[0]}, {expr})'
        else:
            arg_string = ', '.join(self._print(arg) for arg in args)
            return f'Lambda(({arg_string}), {expr})'

    def _print_LatticeOp(self, expr):
        args = sorted(expr.args, key=default_sort_key)
        return expr.func.__name__ + '(%s)' % ', '.join(self._print(arg) for arg in args)

    def _print_Limit(self, expr):
        e, z, z0, dir = expr.args
        if str(dir) == '+':
            return f'Limit({e}, {z}, {z0})'
        else:
            return f"Limit({e}, {z}, {z0}, dir='{dir}')"

    def _print_list(self, expr):
        return '[%s]' % self.stringify(expr, ', ')

    def _print_MatrixBase(self, expr):
        return expr._format_str(self)
    _print_SparseMatrix = \
        _print_MutableSparseMatrix = \
        _print_ImmutableSparseMatrix = \
        _print_Matrix = \
        _print_DenseMatrix = \
        _print_MutableDenseMatrix = \
        _print_ImmutableMatrix = \
        _print_ImmutableDenseMatrix = \
        _print_MatrixBase

    def _print_MatrixElement(self, expr):
        return self._print(expr.parent) + f'[{expr.i}, {expr.j}]'

    def _print_MatrixSlice(self, expr):
        def strslice(x):
            x = list(x)
            if x[2] == 1:
                del x[2]
            if x[1] == x[0] + 1:
                del x[1]
            if x[0] == 0:
                x[0] = ''
            return ':'.join(map(self._print, x))
        return (self._print(expr.parent) + '[' +
                strslice(expr.rowslice) + ', ' +
                strslice(expr.colslice) + ']')

    def _print_Mul(self, expr):

        prec = precedence(expr)

        c, e = expr.as_coeff_Mul()
        if c < 0:
            expr = _keep_coeff(-c, e)
            sign = '-'
        else:
            sign = ''

        a = []  # items in the numerator
        b = []  # items that are in the denominator (if any)

        if self.order != 'none':
            args = expr.as_ordered_factors()
        else:
            # use make_args in case expr was something like -x -> x
            args = Mul.make_args(expr)

        multiple_ones = len([x for x in args if x == 1]) > 1

        # Gather args for numerator/denominator
        for item in args:
            if item.is_commutative and item.is_Pow and item.exp.is_Rational and item.exp.is_negative:
                if item.exp != -1:
                    b.append(Pow(item.base, -item.exp, evaluate=False))
                else:
                    b.append(Pow(item.base, -item.exp))
            elif item.is_Rational and item is not oo:
                if item.numerator != 1 or multiple_ones:
                    a.append(Rational(item.numerator))
                if item.denominator != 1:
                    b.append(Rational(item.denominator))
            else:
                a.append(item)

        a = a or [Integer(1)]

        a_str = [self.parenthesize(x, prec) for x in a]
        b_str = [self.parenthesize(x, prec) for x in b]

        if len(b) == 0:
            return sign + '*'.join(a_str)
        elif len(b) == 1:
            return sign + '*'.join(a_str) + '/' + b_str[0]
        else:
            return sign + '*'.join(a_str) + '/(%s)' % '*'.join(b_str)

    def _print_MatMul(self, expr):
        return '*'.join([self.parenthesize(arg, precedence(expr))
                         for arg in expr.args])

    def _print_HadamardProduct(self, expr):
        return '.*'.join([self.parenthesize(arg, precedence(expr))
                          for arg in expr.args])

    def _print_MatAdd(self, expr):
        return ' + '.join([self.parenthesize(arg, precedence(expr))
                           for arg in expr.args])

    def _print_NaN(self, expr):
        return 'nan'

    def _print_NegativeInfinity(self, expr):
        return '-oo'

    def _print_Order(self, expr):
        if all(p == 0 for p in expr.point) or not len(expr.variables):
            if len(expr.variables) <= 1:
                return 'O(%s)' % self._print(expr.expr)
            else:
                return 'O(%s)' % self.stringify((expr.expr,) + expr.variables, ', ', 0)
        else:
            return 'O(%s)' % self.stringify(expr.args, ', ', 0)

    def _print_Cycle(self, expr):
        """We want it to print as Cycle in doctests for which a repr is required.

        With __repr__ defined in Cycle, interactive output gives Cycle form but
        during doctests, the dict's __repr__ form is used. Defining this _print
        function solves that problem.

        >>> Cycle(1, 2)  # will print as a dict without this method
        Cycle(1, 2)

        """
        return expr.__repr__()

    def _print_Permutation(self, expr):
        from ..combinatorics import Permutation, Cycle
        if Permutation.print_cyclic:
            if not expr.size:
                return 'Permutation()'
            # before taking Cycle notation, see if the last element is
            # a singleton and move it to the head of the string
            s = Cycle(expr)(expr.size - 1).__repr__()[len('Cycle'):]
            last = s.rfind('(')
            if not last == 0 and ',' not in s[last:]:
                s = s[last:] + s[:last]
            return f'Permutation{s}'
        else:
            s = expr.support()
            if not s:
                if expr.size < 5:
                    return 'Permutation(%s)' % str(expr.array_form)
                return f'Permutation([], size={expr.size})'
            trim = str(expr.array_form[:s[-1] + 1]) + f', size={expr.size}'
            use = full = str(expr.array_form)
            if len(trim) < len(full):
                use = trim
            return f'Permutation({use})'

    def _print_TensorIndex(self, expr):
        return expr._print()

    def _print_TensorHead(self, expr):
        return expr._print()

    def _print_Tensor(self, expr):
        return expr._print()

    def _print_TensMul(self, expr):
        return expr._print()

    def _print_TensAdd(self, expr):
        return expr._print()

    def _print_PermutationGroup(self, expr):
        p = ['    %s' % str(a) for a in expr.args]
        return 'PermutationGroup([\n%s])' % ',\n'.join(p)

    def _print_Pi(self, expr):
        return 'pi'

    def _print_PolyElement(self, poly):
        return poly.str(self, PRECEDENCE, '%s**%d', '*')

    def _print_FracElement(self, frac):
        if frac.denominator == 1:
            return self._print(frac.numerator)
        else:
            numer = self.parenthesize(frac.numerator, PRECEDENCE['Add'])
            denom = self.parenthesize(frac.denominator, PRECEDENCE['Atom']-1)
            return numer + '/' + denom

    def _print_Poly(self, expr):
        terms, gens = [], expr.gens

        for monom, coeff in expr.terms():
            s_monom = []

            for i, exp in enumerate(monom):
                if exp > 0:
                    if exp == 1:
                        s_monom.append(self._print(gens[i]))
                    else:
                        s_monom.append(self.parenthesize(gens[i],
                                                         PRECEDENCE['Atom'] - 1) + f'**{exp:d}')

            s_monom = '*'.join(s_monom)

            if coeff.is_Add:
                if s_monom:
                    s_coeff = '(' + self._print(coeff) + ')'
                else:
                    s_coeff = self._print(coeff)
            else:
                if s_monom:
                    if coeff == 1:
                        terms.extend(['+', s_monom])
                        continue

                    if coeff == -1:
                        terms.extend(['-', s_monom])
                        continue

                s_coeff = self._print(coeff)

            if not s_monom:
                s_term = s_coeff
            else:
                s_term = s_coeff + '*' + s_monom

            if s_term.startswith('-'):
                terms.extend(['-', s_term[1:]])
            else:
                terms.extend(['+', s_term])

        if not terms:
            terms.extend(['+', '0'])

        modifier = terms.pop(0)

        if modifier == '-':
            terms[0] = '-' + terms[0]

        format = expr.__class__.__name__ + '(%s, %s'

        from ..polys.polyerrors import PolynomialError

        try:
            format += ', modulus=%s' % expr.get_modulus()
        except PolynomialError:
            format += f", domain='{expr.domain}'"

        format += ')'

        return format % (' '.join(terms), ', '.join(self._print(s) for s in expr.gens))

    def _print_ProductSet(self, p):
        return ' x '.join(self._print(set) for set in p.sets)

    def _print_AlgebraicElement(self, expr):
        return self._print(expr.parent.to_expr(expr))

    def _print_ModularInteger(self, expr):
        return f'{expr.rep} mod {expr.parent.characteristic}'

    def _print_GaloisFieldElement(self, expr):
        from ..domains import ZZ_python
        return 'GF(%s, %s)(%s)' % (expr.parent.characteristic,
                                   expr.mod.set_domain(ZZ_python).to_dense(),
                                   int(expr))

    def _print_Pow(self, expr, rational=False):
        PREC = precedence(expr)

        if not expr.exp.is_Float and expr.exp == Rational(1, 2) and not rational:
            return 'sqrt(%s)' % self._print(expr.base)

        if expr.is_commutative and not expr.exp.is_Float:
            if -expr.exp == Rational(1, 2) and not rational:
                return '1/sqrt(%s)' % self._print(expr.base)
            if expr.exp == -1:
                return '1/%s' % self.parenthesize(expr.base, PREC)

        e = self.parenthesize(expr.exp, PREC)
        if (self.printmethod == '_diofantrepr' and
                expr.exp.is_Rational and not expr.exp.is_Integer):
            # The parenthesized exp should be '(Rational(a, b))' so strip
            # parens, but just check to be sure.
            return f'{self.parenthesize(expr.base, PREC)}**{e[1:-1]}'
        return f'{self.parenthesize(expr.base, PREC)}**{e}'

    def _print_Mod(self, expr):
        PREC = precedence(expr)

        a, b = expr.args
        return f'{self.parenthesize(a, PREC)}%{self.parenthesize(b, PREC)}'

    def _print_MatPow(self, expr):
        PREC = precedence(expr)
        return '%s**%s' % (self.parenthesize(expr.base, PREC),
                           self.parenthesize(expr.exp, PREC))

    def _print_ImmutableDenseNDimArray(self, expr):
        return str(expr)

    _print_ImmutableSparseNDimArray = _print_ImmutableDenseNDimArray

    _print_MutableDenseNDimArray = _print_ImmutableDenseNDimArray
    _print_MutableSparseNDimArray = _print_ImmutableDenseNDimArray

    def _print_Integer(self, expr):
        return str(expr.numerator)

    def _print_Rational(self, expr):
        if expr.denominator == 1:
            return str(expr.numerator)
        else:
            return f'{expr.numerator}/{expr.denominator}'

    def _print_Float(self, expr):
        prec = expr._prec
        if prec < 5:
            dps = 0
        else:
            dps = prec_to_dps(expr._prec)
        if self._settings['full_prec'] is True:
            strip = False
        elif self._settings['full_prec'] is False:
            strip = True
        elif self._settings['full_prec'] == 'auto':
            strip = self._print_level > 1
        else:
            raise NotImplementedError
        rv = mlib.to_str(expr._mpf_, dps, strip_zeros=strip)
        if rv.startswith('-.0'):
            rv = '-0.' + rv[3:]
        elif rv.startswith('.0'):
            rv = '0.' + rv[2:]
        elif rv.startswith('+'):
            rv = rv[1:]
        return rv

    def _print_Relational(self, expr):

        charmap = {
            '==': 'Eq',
            '!=': 'Ne',
        }

        if expr.rel_op in charmap:
            return f'{charmap[expr.rel_op]}({expr.lhs}, {expr.rhs})'

        return '%s %s %s' % (self.parenthesize(expr.lhs, precedence(expr)),
                             self._relationals.get(expr.rel_op) or expr.rel_op,
                             self.parenthesize(expr.rhs, precedence(expr)))

    def _print_RootOf(self, expr):
        p = self._print_Add(expr.expr, order='lex')
        if expr.free_symbols:
            return f'RootOf({p}, {expr.poly.gen}, {expr.index:d})'
        else:
            return f'RootOf({p}, {expr.index:d})'

    def _print_RootSum(self, expr):
        args = [self._print_Add(expr.expr, order='lex')]

        if expr.fun is not S.IdentityFunction:
            args.append(self._print(expr.fun))

        return 'RootSum(%s)' % ', '.join(args)

    def _print_GroebnerBasis(self, basis):
        cls = basis.__class__.__name__

        exprs = [ self._print_Add(arg, order=basis.order)
                  for arg in basis.exprs ]
        exprs = '[%s]' % ', '.join(exprs)

        gens = [ self._print(gen) for gen in basis.gens ]
        domain = "domain='%s'" % self._print(basis.domain)
        order = "order='%s'" % self._print(basis.order)

        args = [exprs] + gens + [domain, order]

        return '%s(%s)' % (cls, ', '.join(args))

    def _print_frozenset(self, s):
        items = sorted(s, key=default_sort_key)

        args = ', '.join(self._print(item) for item in items)
        if args:
            args = '{%s}' % args
        return f'{type(s).__name__}({args})'

    def _print_set(self, expr):
        if expr == set():
            return 'set()'
        return '{%s}' % self.stringify(sorted(expr, key=default_sort_key), ', ')

    def _print_Sum(self, expr):
        def _xab_tostr(xab):
            return self._print((xab[0],) + tuple(xab[1:]))
        L = ', '.join([_xab_tostr(l) for l in expr.limits])
        return f'Sum({self._print(expr.function)}, {L})'

    def _print_Symbol(self, expr):
        return expr.name
    _print_BaseSymbol = _print_Symbol
    _print_MatrixSymbol = _print_Symbol
    _print_RandomSymbol = _print_Symbol

    def _print_Identity(self, expr):
        return 'I'

    def _print_ZeroMatrix(self, expr):
        return '0'

    def _print_str(self, expr):
        return expr

    def _print_tuple(self, expr):
        if len(expr) == 1:
            return '(%s,)' % self._print(expr[0])
        else:
            return '(%s)' % self.stringify(expr, ', ')

    def _print_Tuple(self, expr):
        return self._print_tuple(expr)

    def _print_Monomial(self, expr):
        if expr.gens:
            return '*'.join(['%s**%s' % (gen, exp)
                             for gen, exp in zip(expr.gens, expr)])
        else:
            return self._print_tuple(expr)

    def _print_Transpose(self, T):
        return '%s.T' % self.parenthesize(T.arg, PRECEDENCE['Pow'])

    def _print_Union(self, expr):
        return ' U '.join(self._print(set) for set in expr.args)

    def _print_Complement(self, expr):
        return r' \ '.join(self._print(set) for set in expr.args)

    def _print_Wild(self, expr):
        return expr.name + '_'

    def _print_WildFunction(self, expr):
        return expr.name + '_'

    def _print_Zero(self, expr):
        return '0'

    def _print_BaseScalarField(self, field):
        return field._coord_sys._names[field._index]

    def _print_BaseVectorField(self, field):
        return f'e_{field._coord_sys._names[field._index]}'

    def _print_Differential(self, diff):
        field = diff._form_field
        if hasattr(field, '_coord_sys'):
            return f'd{field._coord_sys._names[field._index]}'
        else:
            return 'd(%s)' % self._print(field)

    def _print_Tr(self, expr):
        # TODO : Handle indices
        return '%s(%s)' % ('Tr', self._print(expr.args[0]))

    def _print_Domain(self, expr):
        return expr.rep


def sstr(expr, **settings):
    """Returns the expression as a string.

    For large expressions where speed is a concern, use the setting
    order='none'.

    Examples
    ========

    >>> sstr(Eq(a + b, 0))
    'Eq(a + b, 0)'

    """
    p = StrPrinter(settings)
    s = p.doprint(expr)

    return s


class StrReprPrinter(StrPrinter):
    """(internal) -- see sstrrepr"""

    def _print_str(self, s):
        return repr(s)


def sstrrepr(expr, **settings):
    """Return expr in mixed str/repr form.

    i.e. strings are returned in repr form with quotes, and everything else
    is returned in str form.

    This function could be useful for hooking into sys.displayhook
    """
    p = StrReprPrinter(settings)
    s = p.doprint(expr)

    return s



"""
This module implements methods for solving equations and inequalities.
"""

from .inequalities import reduce_inequalities
from .ode import dsolve
from .recurr import rsolve
from .solvers import solve


__all__ = 'reduce_inequalities', 'dsolve', 'rsolve', 'solve'



from diofant import FunctionMatrix, Lambda, Matrix, MatrixExpr
from diofant.abc import i, j


__all__ = ()


def test_funcmatrix():
    X = FunctionMatrix(3, 3, Lambda((i, j), i - j))
    assert X[1, 1] == 0
    assert X[1, 2] == -1
    assert X.shape == (3, 3)
    assert X.rows == X.cols == 3
    assert Matrix(X) == Matrix(3, 3, lambda i, j: i - j)
    assert isinstance(X*X + X, MatrixExpr)



"""A module providing information about the necessity of brackets"""

from ..core.function import _coeff_isneg


#: Default precedence values for some basic types.
PRECEDENCE = {
    'Lambda': 1,
    'Xor': 10,
    'Or': 20,
    'And': 30,
    'Relational': 35,
    'Add': 40,
    'Mul': 50,
    'Mod': 50,
    'Pow': 60,
    'Func': 70,
    'Not': 100,
    'Atom': 1000,
    'BitwiseOr': 36,
    'BitwiseAnd': 38,
}

#: A dictionary assigning precedence values to certain classes. These values
#: are treated like they were inherited, so not every single class has to
#: be named here.
PRECEDENCE_VALUES = {
    'Equivalent': PRECEDENCE['Xor'],
    'Xor': PRECEDENCE['Xor'],
    'Implies': PRECEDENCE['Xor'],
    'Or': PRECEDENCE['Or'],
    'And': PRECEDENCE['And'],
    'Add': PRECEDENCE['Add'],
    'Mod': PRECEDENCE['Mod'],
    'Pow': PRECEDENCE['Pow'],
    'Relational': PRECEDENCE['Relational'],
    'Sub': PRECEDENCE['Add'],
    'Not': PRECEDENCE['Not'],
    'factorial': PRECEDENCE['Func'],
    'factorial2': PRECEDENCE['Func'],
    'Function': PRECEDENCE['Func'],
    'NegativeInfinity': PRECEDENCE['Add'],
    'MatAdd': PRECEDENCE['Add'],
    'MatMul': PRECEDENCE['Mul'],
    'MatPow': PRECEDENCE['Pow'],
    'HadamardProduct': PRECEDENCE['Mul'],
    'Fraction': PRECEDENCE['Atom'],
    'Equality': PRECEDENCE['Mul'],
    'Unequality': PRECEDENCE['Mul'],
}

# Sometimes it's not enough to assign a fixed precedence value to a
# class. Then a function can be inserted in this dictionary that takes
# an instance of this class as argument and returns the appropriate
# precedence value.

# Precedence functions


def precedence_Mul(item):
    if _coeff_isneg(item):
        return PRECEDENCE['Add']
    return PRECEDENCE['Mul']


def precedence_Rational(item):
    if item.numerator < 0:
        return PRECEDENCE['Add']
    return PRECEDENCE['Mul']


def precedence_Integer(item):
    if item.numerator < 0:
        return PRECEDENCE['Add']
    return PRECEDENCE['Atom']


def precedence_Float(item):
    if item < 0:
        return PRECEDENCE['Add']
    return PRECEDENCE['Atom']


def precedence_PolyElement(item):
    if item.is_generator:
        return PRECEDENCE['Atom']
    elif item.is_ground:
        return precedence(item.coeff(1))
    elif item.is_term:
        return PRECEDENCE['Mul']
    else:
        return PRECEDENCE['Add']


def precedence_FracElement(item):
    if item.denominator == 1:
        return precedence_PolyElement(item.numerator)
    else:
        return PRECEDENCE['Mul']


#: Sometimes it's not enough to assign a fixed precedence value to a class. Then
#: a function can be inserted in this dictionary that takes an instance of this
#: class as argument and returns the appropriate precedence value.
PRECEDENCE_FUNCTIONS = {
    'Integer': precedence_Integer,
    'Mul': precedence_Mul,
    'Rational': precedence_Rational,
    'Float': precedence_Float,
    'PolyElement': precedence_PolyElement,
    'FracElement': precedence_FracElement,
}


def precedence(item):
    """
    Returns the precedence of a given object.
    """
    mro = item.__class__.__mro__
    for i in mro:
        n = i.__name__
        if n in PRECEDENCE_FUNCTIONS:
            return PRECEDENCE_FUNCTIONS[n](item)
        elif n in PRECEDENCE_VALUES:
            return PRECEDENCE_VALUES[n]
    return PRECEDENCE['Atom']



"""Square-free decomposition algorithms and related tools."""

from .densearith import dmp_mul, dmp_quo, dmp_sub
from .densebasic import (dmp_degree_in, dmp_ground_LC, dmp_ground_p,
                         dmp_inject, dmp_one_p, dmp_raise, dmp_swap,
                         dmp_zero_p)
from .densetools import (dmp_compose, dmp_diff_in, dmp_ground_monic,
                         dmp_ground_primitive)
from .euclidtools import dmp_gcd, dmp_resultant
from .polyerrors import DomainError


def dmp_sqf_p(f, u, K):
    """
    Return ``True`` if ``f`` is a square-free polynomial in ``K[X]``.

    Examples
    ========

    >>> _, x, y = ring('x y', ZZ)

    >>> ((x + y)**2).is_squarefree
    False
    >>> (x**2 + y**2).is_squarefree
    True

    """
    if dmp_ground_p(f, None, u):
        return True
    else:
        g = f
        for i in range(u + 1):
            g = dmp_gcd(g, dmp_diff_in(f, 1, i, u, K), u, K)
            if dmp_ground_p(g, None, u):
                return True
        return False


def dmp_sqf_norm(f, u, K):
    """
    Square-free norm of ``f`` in ``K[X]``, useful over algebraic domains.

    Returns ``s``, ``f``, ``r``, such that ``g(x) = f(x-sa)`` and ``r(x) = Norm(g(x))``
    is a square-free polynomial over K, where ``a`` is the algebraic extension of ``K``.

    Examples
    ========

    >>> _, x, y = ring('x y', QQ.algebraic_field(I))

    >>> (x*y + y**2).sqf_norm()
    (1, x*y - I*x + y**2 - 3*I*y - 2,
     x**2*y**2 + x**2 + 2*x*y**3 + 2*x*y + y**4 + 5*y**2 + 4)

    """
    if not K.is_AlgebraicField:
        raise DomainError('ground domain must be algebraic')

    g = dmp_raise(K.mod.to_dense(), u + 1, 0, K.domain)
    F = dmp_raise([K.one, -K.unit], u, 0, K)

    s = 0

    while True:
        h, _ = dmp_inject(f, u, K, front=True)
        r = dmp_resultant(g, h, u + 1, K.domain)

        if dmp_sqf_p(r, u, K.domain):
            return s, f, r
        else:
            for j in range(u + 1):
                f = dmp_swap(f, 0, j, u, K)
                f = dmp_compose(f, F, u, K)
                f = dmp_swap(f, 0, j, u, K)
            s += 1


def dmp_sqf_part(f, u, K):
    """
    Returns square-free part of a polynomial in ``K[X]``.

    Examples
    ========

    >>> _, x, y = ring('x y', ZZ)

    >>> (x**3 + 2*x**2*y + x*y**2).sqf_part()
    x**2 + x*y

    """
    if K.is_FiniteField:
        _, sqf = dmp_sqf_list(f, u, K)

        g = [K.one]
        for f, _ in sqf:
            g = dmp_mul(g, f, u, K)

        return g

    if dmp_zero_p(f, u):
        return f

    gcd = f
    for i in range(u + 1):
        gcd = dmp_gcd(gcd, dmp_diff_in(f, 1, i, u, K), u, K)
    sqf = dmp_quo(f, gcd, u, K)

    if K.is_Field:
        return dmp_ground_monic(sqf, u, K)
    else:
        return dmp_ground_primitive(sqf, u, K)[1]


def dup_gf_musser_sqf_list(f, K):
    """Compute square-free decomposition of the monic ``f`` in ``GF(q)[x]``.

    References
    ==========

    * :cite:`Geddes1992algorithms`, algorithm 8.3

    """
    n, factors, p = 1, [], K.characteristic
    m = K.order // p

    while not dmp_ground_p(f, None, 0):
        df = dmp_diff_in(f, 1, 0, 0, K)

        if not dmp_zero_p(df, 0):
            g = dmp_gcd(f, df, 0, K)
            h = dmp_quo(f, g, 0, K)
            f = g
            i = 1

            while not dmp_one_p(h, 0, K):
                g = dmp_gcd(f, h, 0, K)
                h = dmp_quo(h, g, 0, K)

                if dmp_degree_in(h, 0, 0) > 0:
                    factors.append((h, i*n))

                f, h, i = dmp_quo(f, g, 0, K), g, i + 1

        d = dmp_degree_in(f, 0, 0) // p
        n *= p

        for i in range(d + 1):
            f[i] = f[i*p]**m
        del f[d + 1:]

    return factors


def dmp_gf_sqf_list(f, u, K):
    """Compute square-free decomposition of the monic ``f`` in ``GF(q)[X]``.

    Examples
    ========

    >>> _, x = ring('x', FF(11))
    >>> f = x**11 + 1

    Note that:

    >>> f.diff()
    0 mod 11

    This phenomenon doesn't happen in characteristic zero. However we can
    still compute square-free decomposition of ``f``:

    >>> f.sqf_list()
    (1 mod 11, [(x + 1 mod 11, 11)])

    """
    if not u:
        return dup_gf_musser_sqf_list(f, K)

    raise NotImplementedError('multivariate polynomials over finite fields')


def dmp_rr_yun0_sqf_list(f, u, K):
    """Compute square-free decomposition of ``f`` in zero-characteristic ring ``K[X]``.

    References
    ==========

    * :cite:`Geddes1992algorithms`, algorithm 8.2
    * :cite:`LeeM2013factor`, algorithm 3.1

    """
    if dmp_ground_p(f, None, u):
        return []

    result, count = [], 1
    qs = [dmp_diff_in(f, 1, i, u, K) for i in range(u + 1)]

    g = f
    for q in qs:
        g = dmp_gcd(g, q, u, K)

    while not dmp_one_p(f, u, K):
        for i in range(u + 1):
            qs[i] = dmp_quo(qs[i], g, u, K)
        f = dmp_quo(f, g, u, K)
        for i in range(u + 1):
            qs[i] = dmp_sub(qs[i], dmp_diff_in(f, 1, i, u, K), u, K)

        g = f
        for q in qs:
            g = dmp_gcd(g, q, u, K)
        if not dmp_one_p(g, u, K):
            result.append((g, count))

        count += 1

    return result


def dmp_sqf_list(f, u, K):
    """
    Return square-free decomposition of a polynomial in ``K[X]``.

    Examples
    ========

    >>> _, x, y = ring('x y', ZZ)

    >>> (x**5 + 2*x**4*y + x**3*y**2).sqf_list()
    (1, [(x + y, 2), (x, 3)])

    """
    if K.is_Field:
        coeff = dmp_ground_LC(f, u, K)
        f = dmp_ground_monic(f, u, K)
    else:
        coeff, f = dmp_ground_primitive(f, u, K)

    if K.is_FiniteField:
        return coeff, dmp_gf_sqf_list(f, u, K)

    return coeff, dmp_rr_yun0_sqf_list(f, u, K)
